from PASTEC.utils.AbstractClassifierAgent import AbstractClassifierAgent
from PASTEC.Decision import Condition, NumericRule

class SSRAdaptator_Agent(AbstractClassifierAgent):

    def __init__(self, env):
        AbstractClassifierAgent.__init__(self, env)
        self._listOfKnownAgents = ["Size_Agent", "SSRtrf_Agent"]
        self.CONDITIONS = {"classif_other":  [Condition([NumericRule(self._environment.getMaxSSRsize(), "<")]),
                                              Condition([NumericRule(float(self._environment.getMinSSRCoverage()), ">")])],
                          "classif_BM":     [],
                          "weighting":      []}            
    
    def getAgentName(self):
        return "SSR"

    def decide(self):
        lOtherConditions = self.CONDITIONS["classif_other"]
        if self._dNumericData and "consensusLength" in self._dNumericData and "ssrCoverage" in self._dNumericData:
            if self._dNumericData["consensusLength"] and self._dNumericData["ssrCoverage"]:
                if (lOtherConditions[0].validate([self._dNumericData["consensusLength"][0]])) or (lOtherConditions[1].validate([self._dNumericData["ssrCoverage"][0]])):
                    self._classification = self.getAgentName()
                    self._CI += 1
        
        normalizedIC = self._normalizeIC()
        return self._classification, normalizedIC
                    
    def makeAppropriateConclusionsForSSRtrf_Agent(self, iDemandAndResult):
        if iDemandAndResult != None and iDemandAndResult.result != None:
            iSSRCollection = iDemandAndResult.result
            self._dNumericData["consensusLength"]=[]
            self._dNumericData["ssrCoverage"]=[]
            
            self._dNumericData["consensusLength"].append(self._environment.getConsensusLength(self._consensusName))
            self._dNumericData["ssrCoverage"].append(iSSRCollection.getSSRCoverage())
            self._conclusionsMatch.append({'SSR': "%.2f" % float(iSSRCollection.getSSRCoverage())})
                
    def makeAppropriateConclusionsForSize_Agent(self, iDemandAndResult):
        if iDemandAndResult != None and iDemandAndResult.result != None:
            iSize = iDemandAndResult.result
            bound = int(self._environment.getMaxSSRsize())
            msg = "<%sbps" % bound
            if iSize.getSize() > bound:
                msg = ">%sbps" % bound
            self._conclusionsSearch.append(msg)
            self._dNumericData["sizeFull"].append(iSize.getSize())
