from PASTEC.Demand import Demand

## Use this class to create a new analyser agent.

class AbstractAnalyseAgent(object):
    
    ## Constructor
    #
    # @param env Instance of Environment class
    # @param projectName string name of project
    #
    def __init__(self, env):
        self._db = env.getDbInstance()
        self._environment = env
        self.setTableName()
        self._listOfKnownAgents = []
        self._setAgentDecisionRules()

    ## Set table name
    #
    def setTableName(self): 
        self._tableName = self._environment.getProjectName() + self.getSuffixTableNameList()[0]
        
    def setDependentAgentsList(self, lAgents):
        self._listOfKnownAgents = lAgents
        
    ## Get suffix table name list
    #
    # @return list of string SUFFIX_TABLE_NAME_LIST
    #
    def getSuffixTableNameList(self):
        return self.SUFFIX_TABLE_NAME_LIST
    
    def getDependentAgentsList(self):
        return self._listOfKnownAgents
    
    def _setAgentDecisionRules(self):
        """sets the current agen'ts decision rules based on info from configuration file"""
        pass#decisionRulesData = self._environment.getAgentDecisionRules(self.__class__.__name__)

        
    ## Analyse accepted demands and add Result on black board results
    #        
    def live(self):
        for consensusName in self._environment.getConsensusNameList():
            iDemand = Demand(self.__class__.__name__, consensusName)
            lOfFeatures = self.getListOfFeaturesToAnalyse(iDemand.consensusName)
            result = self.analyse(iDemand, lOfFeatures)
            self._environment.addResultToBlackBoard(result)
        
    #/!\ FOR TESTS ONLY
    ## Analyse accepted demands and add Result on black board results
    #        
    def liveForTests(self):
        for consensusName in self._environment.getConsensusNameList():
            iDemand = Demand(self.__class__.__name__, consensusName)
            lOfFeatures = self.getListOfFeaturesToAnalyse(iDemand.consensusName)
            result = self.analyse(iDemand, lOfFeatures)
            self._environment.addResultToBlackBoard(result)
                
    ## get size of the consensus
    # @param consensusName string name of TE consensus
    # @return integer size of TE consensus
    #
    def askForConsensusLength(self, consensusName):
        return self._environment.getConsensusLength(consensusName)
    
    ## get useful informations in database
    #
    # @param consensusName string name of consensus
    #
    def getListOfFeaturesToAnalyse(self, consensusName):
        pass
    
    ## Analyse a Demand (return a Result)
    #
    # @param demand a Demand object type (name of Agent and name of sequence to analyse for example)
    #
    def analyse(self, demand):
        pass