import sys, os, optparse, random, tarfile
from galaxy_utils.sequence.fastq import fastqReader, fastqVerboseErrorReader, fastqAggregator, fastqWriter

def stop_err(msg):
	sys.stderr.write("%s\n" % msg)
	sys.exit()

def main():

	input_filename = sys.argv[1]  #a txt file
	input_type = sys.argv[2]
	output_filename = sys.argv[3] #a txt file
	output_type = sys.argv[4]
	force_quality_encoding = sys.argv[5]
	summarize_input = sys.argv[6] == 'summarize_input'
	if force_quality_encoding == 'None':
		force_quality_encoding = None

	inputFileNames = []
	outGroomerNames = []
	tar = tarfile.open(input_filename, "r")
	workingDir = os.getcwd()
	for tarinfo in tar:	
			name = tarinfo.name
			root = os.path.basename(name).split(".")[0]
			tar.extract(name)
			workingDir = os.getcwd()
			inputFileNames.append("%s/%s" % (workingDir, name))
			outGroomerNames.append("%s/%s_outGroomer.fastq" % (workingDir, root))
	tar.close()

   	aggregator = fastqAggregator()
    	for i in range(len(outGroomerNames)):
		out = fastqWriter( open( outGroomerNames[i], 'wb' ), format = output_type, force_quality_encoding = force_quality_encoding )
		read_count = None
		if summarize_input:
	    		reader = fastqVerboseErrorReader
		else:
	    		reader = fastqReader
		for read_count, fastq_read in enumerate( reader( open( inputFileNames[i] ), format = input_type, apply_galaxy_conventions = True ) ):
	    		if summarize_input:
	        		aggregator.consume_read( fastq_read )
	    		out.write( fastq_read )
		out.close()
	    
	if read_count is not None:
	    print "Groomed %i %s reads into %s reads." % ( read_count + 1, input_type, output_type )
	    if input_type != output_type and 'solexa' in [ input_type, output_type ]:
	        print "Converted between Solexa and PHRED scores."
	    if summarize_input:
	        print "Based upon quality and sequence, the input data is valid for: %s" % ( ", ".join( aggregator.get_valid_formats() )  or "None" )
	        ascii_range = aggregator.get_ascii_range()
	        decimal_range =  aggregator.get_decimal_range()
	        print "Input ASCII range: %s(%i) - %s(%i)" % ( repr( ascii_range[0] ), ord( ascii_range[0] ), repr( ascii_range[1] ), ord( ascii_range[1] ) ) #print using repr, since \x00 (null) causes info truncation in galaxy when printed
	        print "Input decimal range: %i - %i" % ( decimal_range[0], decimal_range[1] )        
	else:
	     print "No valid FASTQ reads were provided."


	tar = tarfile.open(output_filename, "w")
	for fileName in outGroomerNames:
		tar.add(os.path.basename(fileName))
	tar.close()



if __name__ == "__main__": main()
