#!/usr/bin/env Rscript

args <- commandArgs(TRUE)
inputfile <- args[1]
outputfile <- args[2]
title <- args[3]
width <- args[4]
height <- args[5]
################################################################
gffRead <- function(gffFile, nrows = -1) {
     cat("Reading ", gffFile, ": ", sep="")
     gff = read.table(gffFile, sep="\t", as.is=TRUE, quote="",
     header=FALSE, comment.char="#", nrows = nrows,
     colClasses=c("character", "character", "character", "integer",  
"integer",
     "character", "character", "character", "character"))
     colnames(gff) = c("seqname", "source", "feature", "start", "end",
             "score", "strand", "frame", "attributes")
     cat("found", nrow(gff), "rows with classes:",
         paste(sapply(gff, class), collapse=", "), "\n")
     stopifnot(!any(is.na(gff$start)), !any(is.na(gff$end)))
     return(gff)
}

getAttributeField <- function (x, field, attrsep = ";") {
     s = strsplit(x, split = attrsep, fixed = TRUE)
     sapply(s, function(atts) {
         a = strsplit(atts, split = "=", fixed = TRUE)
         m = match(field, sapply(a, "[", 1))
         if (!is.na(m)) {
             rv = a[[m]][2]
         }
         else {
             #rv = as.character(NA)
             rv = NaN
         }
         return(as.numeric(rv))#added conversion to numeric
     })
}
################################################################

#map format
#data <- read.table(inputfile, sep="\t", header = FALSE, col.names=c("seqname","chrname","start","end"))
#gff
#data <- read.table(inputfile, sep="\t", header = FALSE, col.names=c("chrname","toolname","elemname","start","end","bla","strand","bli", "extra_data"))
data =  gffRead(inputfile)
#sumNbKmers <- getAttributeField(data$attributes, "nbElements")#"sumNbelements")
attrData <- getAttributeField(data$attributes, "coverage")
starts <- data$start


#png(paste(inputfile,".repetitions.png",sep=""), as.numeric(width), as.numeric(height))
png(outputfile, as.numeric(width), as.numeric(height))

g_range <- range(starts, starts)
g_range2 <- range(attrData, attrData)

plot(starts,attrData, type="o",col = c("blue"), xlab="chr coordinates", ylab="nb kmers",main="Tallymer Coverage", axes=F,ann=T, xlim =g_range , ylim=g_range2)
axis(1, las=1,   at=starts)
axis(2, las=1, at=attrData)

legend("topleft",c("Kmer coverage"),lty=c(1,1),col=c("blue"))
graphics.off()


#df <- data.frame(ID=11:13, FOO=c('a|b','b|c','x|y'))
#foo <- data$extra_data(do.call('rbind', strsplit(as.character(df$FOO),';',fixed=TRUE)))
#within(df, FOO<-data.frame(do.call('rbind', strsplit(as.character(FOO), '|', fixed=TRUE))))
#print(data)

