#!/usr/bin/env python

import user, os, sys, getopt, exceptions, ConfigParser
from commons.core.sql.DbFactory import DbFactory
from commons.core.sql.TableBinSetAdaptator import TableBinSetAdaptator
from commons.core.sql.TablePathAdaptator import TablePathAdaptator
from commons.core.coord.SetUtils import SetUtils

def setup_env():
    if "REPET_PATH" in os.environ.keys():
        sys.path.append( os.environ["REPET_PATH"] )
    else:
        print "*** Error: no environment variable REPET_PATH ***"
        sys.exit(1)
setup_env()

#-----------------------------------------------------------------------------

def help():

    print ""
    print "usage:",sys.argv[0]," [ options ]"
    print "options:"
    print "     -h: this help"
    print "     -i: tables and commands filename [compulsory]. Structure of the command file:"
    print "         one line per command, each field seperated by one tabulation."
    print "         fields in column order:"
    print "         <table_name>: name of the table to insert"
    print "         <prefix>: prefix name"
    print "     -o: resulting path table name [compulsory]"
    print "     -v: verbose"
    print "     -c: configuration file from TEdenovo or TEannot pipeline"
    print "     -H: MySQL host (if no configuration file)"
    print "     -U: MySQL user (if no configuration file)"
    print "     -P: MySQL password (if no configuration file)"
    print "     -D: MySQL database (if no configuration file)"
    print ""

#-----------------------------------------------------------------------------

def add_table( source, dest, name, verbose ):

    print "\n* insert data from %s" % ( source ); sys.stdout.flush()

    destTableAdaptator = TableBinSetAdaptator( iDb, dest )
    srcTableAdaptator = TablePathAdaptator( iDb, source )

    # retrieve the list of the distinct contigs in the source table
    lContigs = srcTableAdaptator.getContig_name()

    # for each contig
    for c in lContigs:

        # retrieve its HSPs in 'set' format
        clist = srcTableAdaptator.getSetList_from_contig( c )

        # split the resulting 'set' list in several 'set' lists according to the ID
        qhash = SetUtils.getDictOfListsWithIdAsKey( clist )

        for key, alist in qhash.items():
            if verbose:
                print "in table '%s', insert path # %s" % ( source, key )
                sys.stdout.flush()
            for i in alist:
                i.name = name + i.name
            destTableAdaptator.insAddSetList( alist )

#----------------------------------------------------------------------------

def length_select( src_table, dest_table, verbose ):

    print "\n* length select"; sys.stdout.flush()

    stableSetAdaptator = TableBinSetAdaptator( iDb, src_table )
    dtableSetAdaptator = TableBinSetAdaptator( iDb, dest_table )

    src_table_tmp = src_table + "_tmp"
    iDb.remove_if_exist( src_table_tmp )

    req = "CREATE TABLE %s SELECT path,chr,start,end,sum(abs(start-end+1))as length FROM %s GROUP BY path;" % ( src_table_tmp, src_table )
    iDb.execute( req )

    req = "SELECT path,chr,start,end FROM %s ORDER BY length DESC, path;" % ( src_table_tmp )
    req = iDb.execute( req )

    result = iDb.fetchall()
    for i in result:
        if verbose:
            print "insert path", i; sys.stdout.flush()
        plist1 = stableSetAdaptator.getSetList_from_num( i[0] )
        dtableSetAdaptator.insDiffSetList( plist1 )

#-----------------------------------------------------------------------------

def assign_conf( ref_table, annot_table ):

    print "\n* "; sys.stdout.flush()

    iDb.set2set_range( ref_table, "tmp" )
    ref_table_range = ref_table + "_range_tmp"
    iDb.set2set_range( annot_table, "tmp" )
    annot_table_range = annot_table + "_range_tmp"

    rtableSetAdaptator = TableBinSetAdaptator( iDb, ref_table_range )
    atableSetAdaptator = TableBinSetAdaptator( iDb, annot_table_range )

    lContigs = atableSetAdaptator.getContig_name()
    for c in lContigs:
        annot = atableSetAdaptator.getSetList_from_contig( c )
        for a in annot:
            print a.name
            print a.name.split("_")
            conf_ann = int(a.name.split("_")[0])
            nameTE = a.name.replace("%d_"%(conf_ann),"",1)
            start = a.getMin()
            end = a.getMax()
            ref = rtableSetAdaptator.getSetList_from_qcoord( c, start, end )
            for r in ref:
                if a.length() - r.length() <= 50 and r.length() >= 100:
                    if a.name != r.name:
                        conf_ref = int(r.name.split("_")[0])
                        if conf_ann < conf_ref:
                            
                            req = 'UPDATE %s SET name="%d_%s" WHERE path=%d' % ( annot_table, conf_ref, nameTE, a.id )
                            iDb.execute( req )
                            conf_ann = conf_ref
            print a.id, nameTE, a.name, conf_ann

#-----------------------------------------------------------------------------

def main():

    outTable = ""
    inFileName = ""
    rename = ""
    verbose = False
    configFileName = ""
    host = ""
    user = ""
    passwd = ""
    dbname = ""

    try:
        opts, args = getopt.getopt(sys.argv[1:],"hi:o:vc:H:U:P:D:")
    except getopt.GetoptError:
        help()
        sys.exit(1)
    for o,a in opts:
        if o == "-h":
            help()
            sys.exit(0)
        elif o == "-i":
            inFileName = a
        elif o == "-o":
            outTable = a
        elif o == "-v":
            verbose = True
        elif o == "-c":
            configFileName = a
        elif o == "-H":
            host = a
        elif o == "-U":
            user = a 
        elif o == "-P":
            passwd = a
        elif o == "-D":
            dbname = a

    if inFileName == "" or outTable == "":
        print "*** Error:missing compulsory options"
        help()
        sys.exit(1)

    if configFileName != "":
        config = ConfigParser.ConfigParser()
        config.readfp( open(configFileName) )
        host = config.get("repet_env","repet_host")
        user = config.get("repet_env","repet_user")
        passwd = config.get("repet_env","repet_pw")
        dbname = config.get("repet_env","repet_db")

    if host == "" and os.environ.get( "REPET_HOST" ) != "":
        host = os.environ.get( "REPET_HOST" )
    if user == "" and os.environ.get( "REPET_USER" ) != "":
        user = os.environ.get( "REPET_USER" )
    if passwd == "" and os.environ.get( "REPET_PW" ) != "":
        passwd = os.environ.get( "REPET_PW" )
    if dbname == "" and os.environ.get( "REPET_DB" ) != "":
        dbname = os.environ.get( "REPET_DB" )

    if host == "" or user == "" or passwd == "" or dbname == "":
        print "*** Error: missing information about MySQL connection"
        sys.exit(1)

    global iDb
#    db = pyRepet.sql.RepetDB.RepetDB( user, host, passwd, dbname )
    iDb = DbFactory.createInstance()

    outTable_tmp = outTable + "_tmp"

    iDb.remove_if_exist( outTable )
    iDb.remove_if_exist( outTable_tmp )

    iDb.create_set( outTable )
    iDb.create_set( outTable_tmp )

    # record all the data from the input tables (given in the input file) in the same 'set' list
    inFile = open( inFileName, "r" )
    lines = inFile.readlines()
    for line in lines:
        if line == "":
            break
        rename = ""
        tok = line[:-1].split("\t")
        inTable, rename = tok
        add_table( inTable, outTable_tmp, rename, verbose )
    inFile.close()

    length_select( outTable_tmp, outTable, verbose )

    assign_conf( outTable_tmp, outTable )

    iDb.remove_if_exist( outTable_tmp )

    iDb.close()

    return 0

#------------------------------------------------------------------------------

if __name__ == '__main__':
    main()
