import yaml
from PASTEC.Decision import PresenceRule, Condition, NumericRule
import os


class DecisionRulesParser(object):
    def __init__(self, decisionRulesFileName):
        self._decisionRulesFileName = decisionRulesFileName
    
    def _determineRuleInfo(self, pattern):
        pattern = pattern.split(" ")
        if len(pattern)==1:
            return ("presence",False)
        if pattern[0] in [">","<","=","<=", ">="]:
            return ("numeric",False)
        if pattern[0] == "!":
            return ("presence",True)

    def getAgentRules(self, agentName):
        analyseAgentFields = [""]
        classifAgentFields = ["INIT_CLASSIF", "CI_MAX", "PATTERN_2_SEARCH_IN_BLAST_RESULTS","CATEGORY_LENGTH_BOUND","CONDITIONS"]
        
        agentRules = {}
        if not os.path.exists(self._decisionRulesFileName):
            raise Exception("decision rules filename (%s) does not exist" % self._decisionRulesFileName)
        with open(self._decisionRulesFileName) as rulesFileHandler:
            # use safe_load instead load
            dataMap = yaml.safe_load(rulesFileHandler)
            agentRule = dataMap.get(agentName,None)
            if (agentRule is None):
                raise Exception("Unkown agent name specified (%s)" % agentName)
            
            agentRules["INIT_CLASSIF"] = agentRule.get("INIT_CLASSIF","")
            agentRules["CI_MAX"] = agentRule.get("CI_MAX",20)
            agentRules["PATTERN_2_SEARCH_IN_BLAST_RESULTS"] = agentRule.get("BLR_PATTERN","")
            agentRules["CATEGORY_LENGTH_BOUND"] = agentRule.get("CATEGORY_LENGTH_BOUND",[])
            
            #SPECIAL CASES
            #agentRules["CATEGORY_LENGTH_ORF"] = agentRule.get("CATEGORY_LENGTH_ORF",{})#TODO: should ORF really be a special case
            agentRules["MIN_BLASTER_COVERAGE"] = agentRule.get("MIN_BLASTER_COVERAGE", 30)
            agentRules["MIN_PROFILES_COVERAGE"] = agentRule.get("MIN_PROFILES_COVERAGE", 75) 
            agentRules["MAX_ERROR_WINDOW"] = agentRule.get("MAX_ERROR_WINDOW", 10) 
            agentRules["TERMINAL_WINDOWS_LENGTH"] = agentRule.get("TERMINAL_WINDOWS_LENGTH", 10)
            
            #for environement
            agentRules["TE_SIGNIF_CATEGORY_ORF_BOUNDS"] = agentRule.get("TE_SIGNIF_CATEGORY_ORF_BOUNDS", [1000, 3000, 100000])
            agentRules["BLAST_ID_WEIGHT"] = agentRule.get("BLAST_ID_WEIGHT", ((50,100,1), (40,50,0.9), (30,40,0.8), (0,30,0.7))) 
            agentRules["PROFILES_SCORE_WEIGHT"] = agentRule.get("PROFILES_SCORE_WEIGHT", [0.8, 1]) 
            agentRules["MAX_CI"] = agentRule.get("MAX_CI", 100)
            
            
            
            CONDITIONS = agentRule.get("CONDITIONS",{})
            
            #print("list of decisions in agent", agent)
            conditionCategories = {}
            for conditionCategory, conditionList in CONDITIONS.items():
                
                #print("Categ", conditionCategory)#, "value",value)
                #print("elements in ", conditionCategory,":",conditionList)
                conditionCategories[conditionCategory] = []
                for conditionData in conditionList:
                    condition = Condition()
                    condition.setWeight(conditionData.get("weight",0))
                    for rule in conditionData.get("if",[]):
                        ruleType, invert = self._determineRuleInfo(rule)
                        if (ruleType == 'presence'):
                            presenceRule = PresenceRule(rule, invert)
                            condition._lPresenceRules.append(presenceRule)
                            #print("presenceRule rule", rule, invert)
                        else:#TODO: watch out for invert
                            rule = rule.split(" ")
                            threshold = float(rule[1]) 
                            operand = rule[0]
                            numericRule = NumericRule(threshold, operand, invert)
                            condition._lLengthRules.append(numericRule)
                            #print("numeric rule", operand, threshold, invert)
                        
                    conditionCategories[conditionCategory].append(condition)
            if not "weighting" in conditionCategories.keys():
                conditionCategories["weighting"]=[]
            agentRules["CONDITIONS"] = conditionCategories
            #print("agentRules",agentRules)
            return agentRules

    def getAllAgentRules(self):
        with open('Decision_rules.yaml') as rulesFileHandler:
            # use safe_load instead load
            dataMap = yaml.safe_load(rulesFileHandler)
            for agentRule in dataMap:
                agentRules = {}
                #print(agentRule)
                agent = agentRule.get("Agent",None)
                agentRules["INIT_CLASSIF"] = agentRule.get("INIT_CLASSIF","")
                agentRules["CI_MAX"] = agentRule.get("CI_MAX",20)
                agentRules["PATTERN_2_SEARCH_IN_BLAST_RESULTS"] = agentRule.get("BLR_PATTERN","")
                agentRules["CATEGORY_LENGTH_BOUND"] = agentRule.get("CATEGORY_LENGTH_BOUND",[])
                CONDITIONS = agentRule.get("CONDITIONS",None)
                
                #print("list of decisions in agent", agent)
                conditionCategories = {}
                for conditionCategory, conditionList in CONDITIONS.items():
                    
                    #print("Categ", conditionCategory)#, "value",value)
                    #print("elements in ", conditionCategory,":",conditionList)
                    conditionCategories[conditionCategory] = []
                    for conditionData in conditionList:
                        condition = Condition()
                        condition.weight=conditionData.get("weight",0)
                        for rule in conditionData.get("if",[]):
                            ruleType, invert = self._determineRuleInfo(rule)
                            if (ruleType == 'presence'):
                                presenceRule = PresenceRule(rule, invert)
                                condition._lPresenceRules.append(presenceRule)
                                print("presenceRule rule", rule, invert)
                            else:#TODO: watch out for invert
                                rule = rule.split(" ")
                                threshold = float(rule[1]) 
                                operand = rule[0]
                                numericRule = NumericRule(threshold, operand, invert)
                                condition._lLengthRules.append(numericRule)
                                print("numeric rule", operand, threshold, invert)
                            
                        conditionCategories[conditionCategory].append(condition)
               
                    
                #print(conditionCategories)    
                        #print("weight %d, %s" % (condition.weight, " ".join(condition._lLengthRules)))
                #print("initialClassification",initialClassification, "maxCI",maxCI, "PATTERN_2_SEARCH_IN_BLAST_RESULTS",PATTERN_2_SEARCH_IN_BLAST_RESULTS, "CATEGORY_LENGTH_BOUND", CATEGORY_LENGTH_BOUND)
            #print(dataMap)


#def presenceRule_representer(dumper, data):
#    return dumper.represent_scalar(u'!presenceRule', u'%sd%s' % data)
#
#def presenceRule_constructor(loader, node):
#    value = loader.construct_scalar(node)
#    a, b = map(int, value.split('d'))
#    return PresenceRule(a, b)
#
#yaml.add_representer(PresenceRule, presenceRule_representer)
#yaml.add_constructor(u'!presenceRule', presenceRule_constructor)

#print yaml.dump({'gold': PresenceRule("^Tase$",False)})
#print yaml.load("""{pattern: "^Tase$", invert: true}""")


