from commons.core.sql.TableMapAdaptator import TableMapAdaptator
from commons.core.sql.TablePathAdaptator import TablePathAdaptator
from PASTEC.utils.AbstractAnalyseAgent import AbstractAnalyseAgent
from PASTEC.MatchingSubject import MatchingSubject
from PASTEC.ProteinDomainMatch import ProteinDomainMatch
from PASTEC.DemandAndResult import DemandAndResult
import re
import sys

## This agent search in the database the significant profiles and their coverage in TE consensus (OTHER profiles)
#
class Profiles_Agent(AbstractAnalyseAgent):
 
    SUFFIX_TABLE_NAME_LIST = ["_Profiles_path", "_ProfilesDB_map"]   
    
    def __init__(self, env):
        AbstractAnalyseAgent.__init__(self, env)
        self._filterRuleOnProfileType = self._isOTHERInSeqName
        self._lProfilesDetailed = None 

    def getListOfFeaturesToAnalyse(self, consensusName):
        profiles_path = TablePathAdaptator(self._db, self._tableName)
        lPath = profiles_path.getPathListFromQuery(consensusName)
        lPath_other = self._filterOnProfilesType(lPath)
        lProfiles = []
        for path in lPath_other:
            profile = MatchingSubject(path)
            profile.setProteinDomainLength(self._getProfilesLength(path))
            lProfiles.append(profile)            
        return lProfiles  
               
    def analyse(self, demand, lProfiles):
        consensusLength = self.askForConsensusLength(demand.consensusName)
        self._lProfilesDetailed = self._chainMethodsCallToComputeLProfileDetailed(lProfiles)
        self.calculOfProfilesFrameComparedWithConsensus(self._lProfilesDetailed, consensusLength)
        return self.createResult(demand)
        
    def createResult(self, demand):
        lProteinDomainMatchs = []
        for dProfile in self._lProfilesDetailed:
            iProteinDomainMatch = ProteinDomainMatch(dProfile["name"], dProfile["coverage"], dProfile["start"], re.sub("Frameshifted", "", dProfile['strand']))
            lProteinDomainMatchs.append(iProteinDomainMatch)
        return DemandAndResult(demand, lProteinDomainMatchs)
   
    def _chainMethodsCallToComputeLProfileDetailed(self, lProfiles):
        lPathSorted = self._sortListOfPathBySubjectSeqName(lProfiles)
        lProfilesDict = self.indexCalculationWithGatherCutOffAndQueryCoordCumulationIfOverlap(lPathSorted)
        sortedLProfilesDict = self._sortListOfProfilesDictByStartAndByEnd(lProfilesDict)
        finalLProfilesDict = self.sumIndexAndSumCoverageForIdenticalProfiles(sortedLProfilesDict)  
        return self.removeProfilesWithBadIndex(finalLProfilesDict)
    
    def indexCalculationWithGatherCutOffAndQueryCoordCumulationIfOverlap(self, lProfiles):
        lProfilesDict = []
        currentPosition = 0
        previousProfile = None
        for iPath in lProfiles:
            #select according to Evalue
            if iPath.e_value < self._environment.getMaxProfilesEvalue():
                matchLengthOnSubject = iPath.getLengthOnSubject()
                scoreWeight = self._chooseWeightAccordingToGatherCutOffAndMatchScore(iPath)
                strand = iPath.getSubjectStrand()
                
                queryStart = iPath.range_query.start
                queryEnd = iPath.range_query.end
                profilesOverlapLength = 0
                if currentPosition > 0 :
                    overlapOnQuery = 0
                    previousStrand = ""
                    previousOnlyStrand = ""
                    previousMatchOfCurrentProfile = self._getPreviousMatchOfCurrentProfiles(lProfilesDict, iPath.range_subject.seqname)
                    if previousMatchOfCurrentProfile != {}:
                        previousStrand = previousMatchOfCurrentProfile['strand']
                        previousOnlyStrand = re.sub("Frameshifted", "", previousStrand)
                    if previousMatchOfCurrentProfile != {} and strand == previousOnlyStrand:
                        profilesOverlapLength = self._computeOverlapBetweenSubjects(iPath, previousProfile)
                        overlapOnQuery = self._overlapCalculationOnQuery(iPath, previousMatchOfCurrentProfile)
                    if overlapOnQuery != 0 and previousMatchOfCurrentProfile != {} and strand == previousOnlyStrand :
                        strand = strand + "Frameshifted"
                        queryStart = min(previousMatchOfCurrentProfile['start'], iPath.range_query.start)
                        queryEnd = max(previousMatchOfCurrentProfile['end'], iPath.range_query.end)
            
                percentOfCoverage = self._computeCoverage(iPath._length, matchLengthOnSubject, profilesOverlapLength)
                lProfilesDict.append({'name': iPath.range_subject.seqname, 'strand': strand, 'start': queryStart, 'end': queryEnd, 'index': round(percentOfCoverage * scoreWeight, 2), 'realCoverage': percentOfCoverage})
            
                previousProfile = iPath 
            currentPosition += 1
        return lProfilesDict
                
    def sumIndexAndSumCoverageForIdenticalProfiles(self, profiles_resultsSummary):
        if len(profiles_resultsSummary) == 0:
            return []
        
        dProfilesName2TupleOfStrand = {}
        for dict in profiles_resultsSummary:
            profilesName = dict['name']
            profilesStrand = re.sub("Frameshifted", "", dict['strand'])
            compositeKey = profilesName + profilesStrand
            
            if not dProfilesName2TupleOfStrand.has_key(compositeKey):
                inDProfilesName2TupleOfStrand = dict 
                inDProfilesName2TupleOfStrand['coverage'] = inDProfilesName2TupleOfStrand['realCoverage']
                del inDProfilesName2TupleOfStrand['realCoverage']
                dProfilesName2TupleOfStrand[compositeKey] = inDProfilesName2TupleOfStrand
            else:
                dProfilesName2TupleOfStrand[compositeKey]['coverage'] += dict['realCoverage']
                dProfilesName2TupleOfStrand[compositeKey]['index'] += dict['index']
                dProfilesName2TupleOfStrand[compositeKey]['end'] = dict['end']
        return sorted(dProfilesName2TupleOfStrand.values(), key=lambda k: (k['start'], k['end']))
    
    def _sortMatchingSubjectListBySubjectSeqNameAndQueryStartQueryEnd(self, lMatchingSubjects):
        return sorted(lMatchingSubjects, key=lambda k: (k.range_subject.seqname, k.range_query.start, k.range_query.end))

    # input : lMatchingSubjects only for a same profile name, sort by start and end. 
    def createProteinDomainMatchList(self, lMatchingSubjects):
        lPDM = []
        iPDM = None
        #TODO: previousMatchingSubject = None...
        previousMatchingSubject = MatchingSubject()
        for iMatchingSubject in lMatchingSubjects:
            strand = iMatchingSubject.getSubjectStrand()
            scoreWeight = self._chooseWeightAccordingToGatherCutOffAndMatchScore(iMatchingSubject)
            if strand == previousMatchingSubject.getSubjectStrand() and iMatchingSubject.isQueryOverlapping(previousMatchingSubject):
                self._mergeOverlappingProfiles(iPDM, previousMatchingSubject, iMatchingSubject, strand, scoreWeight)
            else:
                iPDM = self._createProteinDomainMatch(lPDM, iMatchingSubject, strand, scoreWeight)
            previousMatchingSubject = iMatchingSubject
        return lPDM

    def _mergeOverlappingProfiles(self, iPDM, previousMatchingSubject, iMatchingSubject, strand, scoreWeight):
        subjectOverlapLength = self._computeOverlapBetweenSubjects(iMatchingSubject, previousMatchingSubject)
        coverage = self._computeCoverage(iMatchingSubject.getProteinDomainLength(), iMatchingSubject.getLengthOnSubject(), subjectOverlapLength)
        iPDM.increaseCoverage(coverage)
        iPDM.computeIndex(scoreWeight)
        iPDM.setEnd(iMatchingSubject.range_query.end)
#        iPDM.setStrand("%sFrameshifted" % strand)

    def _createProteinDomainMatch(self, lPDM, iMatchingSubject, strand, scoreWeight):
        #not coverage = self._computeCoverage(consensusLength, iMatchingSubject.getLengthOnSubject(), 0) => because coverage on profile, not on consensus (coding info)
        coverage = self._computeCoverage(iMatchingSubject.getProteinDomainLength(), iMatchingSubject.getLengthOnSubject(), 0)
        iPDM = ProteinDomainMatch(iMatchingSubject.range_subject.seqname, start = iMatchingSubject.range_query.start, end = iMatchingSubject.range_query.end, strand = strand, coverage = coverage)
        iPDM.computeIndex(scoreWeight)
        lPDM.append(iPDM)
        return iPDM
       
    def _computeCoverage(self, length_profile, length_match, overlap):
        coverage = (float(length_match - overlap) * 100) / float(length_profile)
        return round(coverage, 2)

    def _chooseWeightAccordingToGatherCutOffAndMatchScore(self, iMatchingSubject):
        tabTmp = iMatchingSubject.range_subject.seqname.split("_")
        gatherCutOff = tabTmp[-1]
        if gatherCutOff == "NA" or float(iMatchingSubject.score) < float(gatherCutOff):
            scoreWeight = self._environment.getProfilesScoreWeight()[0]
        else:
            scoreWeight = self._environment.getProfilesScoreWeight()[1]
        return scoreWeight

    def _computeOverlapBetweenSubjects(self, profiles_result, previousMatchOfCurrentProfile):
        maxProfile1Coord = max(profiles_result.range_subject.start, profiles_result.range_subject.end)
        maxProfile2Coord = max(previousMatchOfCurrentProfile.range_subject.start, previousMatchOfCurrentProfile.range_subject.end)
        minProfile1Coord = min(profiles_result.range_subject.start, profiles_result.range_subject.end)
        minProfile2Coord = min(previousMatchOfCurrentProfile.range_subject.start, previousMatchOfCurrentProfile.range_subject.end)
        overlapLength = self._computeOverlap(maxProfile1Coord, maxProfile2Coord, minProfile1Coord, minProfile2Coord)
        return overlapLength

    def _computeOverlap(self, maxCoord1, maxCoord2, minCoord1, minCoord2):
        return max(0, min(maxCoord1, maxCoord2) - max(minCoord1, minCoord2) + 1)

    def removeProfilesWithBadIndex(self, lProfiles):
        lDProfilesFinal = []
        for dCurrentProfile in lProfiles:
            if dCurrentProfile['index'] >= self._environment.getMinProfilesIndex():
                lDProfilesFinal.append(dCurrentProfile)
        return sorted(lDProfilesFinal, key=lambda k: (k['start'], k['end']))
    
    def _addKeyTypeAndRemoveKeyStrand(self, lProfilesDetailed):
        for dCurrentProfile in lProfilesDetailed:
            profileType = self._getProfileType(dCurrentProfile)
            dCurrentProfile['type'] = profileType
            del dCurrentProfile['strand']

    def _getProfilesLength(self, profilePath):
        profilesMapTableName = self._environment.getProjectName() + self.SUFFIX_TABLE_NAME_LIST[1]
        tmA = TableMapAdaptator(self._db, profilesMapTableName)
        lMap = tmA.getMapListFromSeqName(profilePath.range_subject.seqname)
        
        if lMap == []:
            print "Profile " + profilePath.range_subject.seqname + " is not in database\n"
            sys.exit(1)
            
        profilLength = lMap[0].end
        return profilLength

    def _getPreviousMatchOfCurrentProfiles(self, lProfiles_resultsSummary, profiles_resultName):
        lenLProfiles = len(lProfiles_resultsSummary)
        if lenLProfiles == 0:
            return {}
        dPreviousResult = lProfiles_resultsSummary [lenLProfiles - 1]
        if dPreviousResult['name'] == profiles_resultName:
            return dPreviousResult
        return {}

    def _sortListOfProfilesDictByStartAndByEnd(self, lprofiles_resultsSummary):
        return sorted(lprofiles_resultsSummary, key=lambda k: (k['start'], k['end']))
    
    def _sortListOfPathBySubjectSeqName(self, lPath):
        return sorted(lPath, key=lambda k: k.range_subject.seqname)
    
    def _overlapCalculationOnQuery(self, profiles_result, previousMatchOfCurrentProfile):
        maxQuery1Coord = max(profiles_result.range_query.start, profiles_result.range_query.end)
        maxQuery2Coord = max(previousMatchOfCurrentProfile['start'], previousMatchOfCurrentProfile['end'])
        minQuery1Coord = min(profiles_result.range_query.start, profiles_result.range_query.end)
        minQuery2Coord = min(previousMatchOfCurrentProfile['start'], previousMatchOfCurrentProfile['end'])
        return self._computeOverlap(maxQuery1Coord, maxQuery2Coord, minQuery1Coord, minQuery2Coord)

    def _getProfileType(self, profiles_resultsSummaryFinalItem1):
        tabTmp1 = profiles_resultsSummaryFinalItem1['name'].split("_")
        profileType = tabTmp1[-2]
        return profileType
    
    def _filterOnProfilesType(self, lPath):
        lPath_other = []
        for path in lPath:
            if self._filterRuleOnProfileType(path.range_subject.seqname):
                lPath_other.append(path)
        return lPath_other

    def _isOTHERInSeqName(self, seqname):
        return re.search('_OTHER_', seqname)
    
    def calculOfProfilesFrameComparedWithConsensus(self, listProfilesDetailed, currentConsensusLength):
        pass