import os
import sys
from commons.core.seq.Bioseq import Bioseq
from commons.core.seq.BioseqDB import BioseqDB
from denovo_pipe.DawgLauncher import DawgLauncher
from denovo_pipe.GenomeGrammar import GenomeGrammar

if not os.environ.has_key("REPET_DATA"):
    msg = "ERROR: environment variable REPET_DATA must be set"
    sys.stderr.write( "%s\n" % msg )
    sys.exit(1)

TE_FILE_FOR_SCENARIO = ("%s/%s/%s") % (os.environ["REPET_DATA"], "TEdenovo/simulation/", "G2-edited-DAWG.fa.cons")



class Scenario00(object):
    
    def __init__(self):
        self._clean = True
        self._verbose = 0
        self._teFile = TE_FILE_FOR_SCENARIO
        self._teCopiesPrefix = "%s_copies" % (self._teFile)
        self._seedForBackboneGeneration = None
    
    def setNbTe(self, nbTe):
        self._nbTe = nbTe

    def setSeedForBackboneGeneration(self, seedForBackoneGeneration):
        self._seedForBackboneGeneration = seedForBackoneGeneration
    
    def setSeedForRandomizeProbas(self, seedForRandomizeProbas):
        self._seedForRandomizeProbas = seedForRandomizeProbas
    
    def setClean(self, clean):
        self._clean = clean

    def setVerbose(self, verbose):
        self._verbose = verbose
        
    def generateTEs (self, nbTeCopies):
        if self._verbose > 0:
            print "generate TEs ..."
             
            sys.stdout.flush()
        teFileHandler = open (self._teFile)
        
        iBsIn = Bioseq()
        iBsIn.read(teFileHandler)
        teFileHandler.close()
        
        if self._verbose > 0:
            print "TE orginal sequence is: %s " % iBsIn.header
        
        teCopiesFileHandler = open ("%s.fa" % self._teCopiesPrefix, "w")
        for i in range(nbTeCopies):
            iBsIn.header = "TECopy%s" % i
            iBsIn.write(teCopiesFileHandler)
        teCopiesFileHandler.close()
        return BioseqDB("%s.fa" % self._teCopiesPrefix)

    def getGenomeGrammarConfigFile(self):
        dSymbol2Proba = { "epsilon": 0.00025,
                         "t": 0.0125
                        }
        dSymbol2Proba[ "o" ] = 1 - reduce( lambda x, y: x+y, dSymbol2Proba.values() )
        configFile = "genomeGrammar.cfg"
        configFileHandler = open(configFile, "w")
        configFileHandler.write("[transition_probabilities]\n")
        configFileHandler.write( "o:%f\n" % ( dSymbol2Proba[ "o" ] ) )
        configFileHandler.write( "t:%f\n" % ( dSymbol2Proba[ "t" ] ) )
        configFileHandler.write( "epsilon:%f\n" % ( dSymbol2Proba[ "epsilon" ] ) )
        configFileHandler.close()
        return configFile
    
    def setBsDBInstance(self, iGenomeSimulator):
        iGenomeSimulator._iBsDbTe = BioseqDB("%s.fa" % self._teCopiesPrefix)

    def getStarTree(nbTeCopies, branchLength=0.2):
        tree = "("
        tree += "TEcopy_1:%f" % ( branchLength )
        for i in xrange(1,nbTeCopies):
            tree += ",TEcopy_%i:%f" % ( i+1, branchLength )
        tree += ");"
        return tree
    getStarTree = staticmethod(getStarTree)
    
    def generateGenomeBackbone(self):
        gg = GenomeGrammar()
        return gg.generateBackboneWithSimulations(self._nbTe, self._seedForBackboneGeneration, self._seedForRandomizeProbas)    

class Scenario01 (Scenario00):

    def __init__(self):
        self._nbTe = None
        self._seedForRandomizeProbas = None
        Scenario00.__init__(self)
        
    
    def setDawgConfigFile(self, dawgConfigFile):
        self._dawgConfigFile = dawgConfigFile
 
    def getDawgConfigFile (self):
        return self._dawgConfigFile      
        
    def generateTEs(self, nbTeCopies):
     
        if self._verbose > 0:
            print "generate TEs with Dawg..."
            sys.stdout.flush()
            
        dl = DawgLauncher()
        dl._keepAlignedFasta = True
        dl._configFile = self.getDawgConfigFile()
        dl._seqFile = self._teFile 
        dl._outPrefix = "%s" % self._teCopiesPrefix 
        dl._verbose = self._verbose
        dl.run()
    
        if self._verbose > 0:
            print "TEs generated sequences:%s.fa" % dl._outPrefix

        return BioseqDB("%s.fa" % self._teCopiesPrefix)


