import re
from commons.core.sql.TableSetAdaptator import TableSetAdaptator
from commons.core.coord.Set import Set
from PASTEC.utils.AbstractAnalyseAgent import AbstractAnalyseAgent
from PASTEC.DemandAndResult import DemandAndResult
from PASTEC.TR import TR

## Agent searching tandem terminal or not terminal repeats in database.
#Warning : no threshold for length or identity. Necessary ?
class TRSearch_Agent(AbstractAnalyseAgent):
    
    SUFFIX_TABLE_NAME_LIST = ["_TR_set"]
    
    def __init__(self, env):
        AbstractAnalyseAgent.__init__(self, env)
   
    def _setAgentDecisionRules(self):
        """sets the current agen'ts decision rules based on info from configuration file"""
        decisionRulesData = self._environment.getAgentDecisionRules(self.__class__.__name__)
        self.TERMINAL_WINDOWS_LENGTH = decisionRulesData["TERMINAL_WINDOWS_LENGTH"]
        
    ## Filter sets and format them
    #
    # @param demand Demand instance
    # @param lSet list of set instances 
    # @return result DemandAndResult instance
    #
    def analyse(self, demand, lSet):
        consensusLength = self.askForConsensusLength(demand.consensusName)
        lSortedSets = self.sortSetsListByIdAndConsensusNameAndStartEnd(lSet)
        lTR = []
        previousTRSet = Set()
        for currentTRSet in lSortedSets:
            if currentTRSet.id == previousTRSet.id:
                TRLength = (currentTRSet.getLength() + previousTRSet.getLength()) / 2
                if re.match("Rep\d+.*LTR.*", previousTRSet.name):
                    iTR = TR("termLTR", TRLength)
                    self._addTRInfoInResult(lTR, previousTRSet, currentTRSet, iTR, consensusLength)
                if re.match("Rep\d+.*termTIR.*", previousTRSet.name):
                    iTR = TR("termTIR", TRLength)
                    self._addTRInfoInResult(lTR, previousTRSet, currentTRSet, iTR, consensusLength)
            previousTRSet = currentTRSet
        result = DemandAndResult(demand, lTR)   
        return result 

    ## get the list of features useful for TRSearchAgent
    #
    # @param consensusName a string name of consensus
    # @return lSet list of set
    # @return consensusLength int length of the consensus
    #
    def getListOfFeaturesToAnalyse(self, consensusName):
        tSetAdaptator = TableSetAdaptator(self._db, self._tableName)
        lSet = tSetAdaptator.getSetListFromSeqName(consensusName)
        return lSet
    
    def _areTRterminal(self, TRset, consensusLength):
        startWindow = 1 + self.TERMINAL_WINDOWS_LENGTH
        endWindow = consensusLength - self.TERMINAL_WINDOWS_LENGTH
        return  TRset.start < startWindow or TRset.end < startWindow \
            or TRset.start > endWindow or TRset.end > endWindow

    def _addTRInfoInResult(self, lTR, previousTRSet, currentTRSet, iTR, consensusLength):
        nonTermTRType = "non-%s" % iTR.getType()
        if not self._isAlreadyInList(iTR.getType(), lTR) and self._areTRterminal(previousTRSet, consensusLength) and self._areTRterminal(currentTRSet, consensusLength):
            lTR.append(iTR)
        elif not self._isAlreadyInList(nonTermTRType, lTR) :
            iTR.setType(nonTermTRType)
            lTR.append(iTR)

    #TODO: don't check if it's exactly the same (same length, same position). Necessary ?
    def _isAlreadyInList(self, TRType, lTR): 
        isInList = False
        for iTR in lTR:
            if TRType == iTR.getType():
                isInList = True
        return isInList
    
    def sortSetsListByIdAndConsensusNameAndStartEnd(self, lSets):
        return sorted(lSets, key=lambda k: (k.getId(), k.getSeqname(), k.getStart(), k.getEnd()))