#!/usr/bin/env python

import os
import sys
import getopt
from pyRepet.parser.Parser import Parser
from pyRepet.launcher.programLauncher import programLauncher

#------------------------------------------------------------------------------

def help():

    """
    Give the list of the command-line options.
    """

    print ""
    print "usage:",sys.argv[0],"[ options ]"
    print "options:"
    print "     -h: this help"
    print "     -p: project name"
    print "     -f: fasta file used as query by Blaster"
    print "     -a: 'align' file (output from Blaster)"
    print "     -c: clean the working directory"
    print "     -v: verbose (default=0/1)"
    print ""

#------------------------------------------------------------------------------

def main():

    """
    This program allows to launch Piler easily in the REPET framework.
    """

    projectName = ""
    #faFileName = ""
    alignFileName = ""
    cleanDir = "no"
    verbose = 0

    try:
        options, args = getopt.getopt(sys.argv[1:],"hp:f:a:cv:")
    except getopt.GetoptError:
        help()
        sys.exit(1)
    for o,a in options:
        if o == "-h":
            help()
            sys.exit(0)
        elif o == "-p":
            projectName = a
        elif o == "-f":
            faFileName = a
        elif o == "-a":
            alignFileName = a
        elif o == "-c":
            cleanDir = "yes"
        elif o == "-v":
            verbose = int(a)

    if projectName == "" or alignFileName == "":
        print "*** Error: compulsory options are missing"
        help()
        sys.exit(1)

    if verbose > 0:
        print "\nbeginning of %s" % (sys.argv[0].split("/")[-1])
        sys.stdout.flush()

    #--------------------------------------------------------------------------
##############################################################################COMMENTED OUt FEATURE#######################################
    
##    if verbose > 0:
##        print "prepare input file"; sys.stdout.flush()
##    if os.path.exists( os.environ["REPET_PATH"] + "/bin/align2piler" ):
##        prg = os.environ["REPET_PATH"] + "/bin/align2piler"
##        cmd = prg
##        cmd += " -i %s" % ( alignFileName )
##        cmd += " -o %s.gff" % ( alignFileName )
##        cmd += " -v %i" % ( verbose - 1 )
##        log = os.system( cmd )
##        if log != 0:
##            print "*** Error: %s returned %i" % ( prg, log )
##            sys.exit(1)
##    else:
##        parser = Parser( alignFileName, alignFileName + ".gff" )
##        parser.align2piler()
    
##############################################################################COMMENTED OUt FEATURE#######################################
    #if verbose > 0:
    #   print "Piler is running..."; sys.stdout.flush()
    #pL = programLauncher( alignFileName + ".gff" )
    #pL.launchPiler( projectName + "_Piler-trs.gff", projectName + "_Piler-trs_piles.gff", projectName + "_Piler-trs_images.gff" )

#--------------------------------------------------------------------------

    if verbose > 0:
        print "Piler-TA is running..."
        sys.stdout.flush()
    pL = programLauncher( projectName + "_Piler-trs_images.gff" )
    pL.launchPilerTA( projectName + "_Piler-tan.gff", projectName + "_Piler-tan_motif.gff", projectName + "_Piler-tan_pyr.gff" )

    #--------------------------------------------------------------------------

    if verbose > 0:
        print "convert results into 'map' format"
        sys.stdout.flush()
    parser = Parser( projectName + "_Piler-tan.gff", projectName + "_Piler-tan.gff.info" )
    parser.pilerTA2grouperMap()
#
#   if cleanDir == "yes":
#       os.system( "rm -f *tan_*.gff" )

    #--------------------------------------------------------------------------

    if verbose > 0:
        print "generate fasta from map"
        sys.stdout.flush()
    prg = os.environ["REPET_PATH"] + "/bin/map2db"
    cmd = prg
    cmd += " " + projectName + "_Piler-tan_motif.gff.grp.map"
    cmd += " " + faFileName
    log = os.system( cmd )
    if log != 0:
        print "*** Error: %s returned %i" % ( prg, log )
        sys.exit(1)

    os.system( "mv " + projectName + "_Piler-tan_motif.gff.grp.map.flank_size0.fa " +  projectName + "_Piler-tan.fa" )

    #--------------------------------------------------------------------------

    if verbose > 0:
        print "\n%s finished successfully\n" % (sys.argv[0].split("/")[-1])
        sys.stdout.flush()

    return 0

#------------------------------------------------------------------------------
#------------------------------------------------------------------------------

if __name__ == '__main__':
    main()
