#!/usr/bin/env python

import sys
import os
import time
from commons.core.utils.RepetOptionParser import RepetOptionParser

def main():
    
    description = "Launches SATannot pipeline"
    parser = RepetOptionParser(description = description, version = "v1.0")
    parser.add_option("-C", "--config",  dest = "configFileName", action="store", type="string", default = "SATannot.cfg", help = "configuration file (default = SATannot.cfg)")
    parser.add_option("-S", "--step",    dest = "steps",          action="store", type="string", default = "123456",       help = "step : 1/2/3/4/5/6 (default = 123456)")
    parser.add_option("-v", "--verbose", dest = "verbose",        action="store", type="int",    default = 0,              help = "verbose (default = 0)")
    options = parser.parse_args()[0]

    configFileName = options.configFileName
    steps = options.steps
    verbose = options.verbose

    if configFileName == "":
        print "ERROR: missing configuration file (-C)"
        help()
        sys.exit(1)
        
    if not os.path.exists( configFileName ):
        print "ERROR: file '%s' doesn't exist" % configFileName
        help()
        sys.exit(1)
        
    if steps == "":
        print "ERROR: missing step(s) (-S)"
        help()
        sys.exit(1)
        
    print "START time: %s" % time.strftime("%Y-%m-%d %H:%M:%S")
    sys.stdout.flush()
    
    for stepNb in steps:
        cmd = "SATannot.py -C %s -S %s -v %i" % (configFileName, stepNb, verbose)
        returnValue = os.system(cmd)
        if returnValue != 0:
            print "ERROR: command '%s' returned %i" % (cmd, returnValue)
            sys.exit(1)
            
    print "END time: %s" % time.strftime("%Y-%m-%d %H:%M:%S")
    sys.stdout.flush()
    
    return 0
    
if __name__== "__main__":
    main()
