#!/usr/bin/env python

import os
startupfilename = os.environ.get('PYTHONSTARTUP')
if startupfilename and os.path.isfile(startupfilename):
    execfile(startupfilename)

import sys
import getopt
from commons.core.coord.Set import Set
from commons.core.coord.Align import Align
from commons.core.coord.SetUtils import SetUtils
from commons.core.sql.DbFactory import DbFactory
from SegDup_pipe.ExtractTable import type_table
from SegDup_pipe.ExtractTable import extract_repeat_overlap
from SegDup_pipe.ExtractTable import extract_numberRepeat_end
        
def detect_overlap (table,query,repet):
    newresult=[]
    type=type_table(table,repet)
    result=extract_repeat_overlap(type,table,query.seqname,query.start,query.end,repet)
    i=0
    for r in result:
        if type=="path":
            newresult.append((int(r[0]),r[4],r[1],int(r[2]),int(r[3])))
        if type=="align":
            newresult.append((i,r[4],r[1],int(r[2]),int(r[3])))
        if type=="map":
            newresult.append((i,r[0],r[1],int(r[2]),int(r[3])))
        if type=="set":
            newresult.append((int(r[0]),r[1],r[2],int(r[3]),int(r[4])))  
        i=i+1
    return newresult



def over_set_list(list_tables,query,repet):
    subject_list=[]
    subject_list_merge=[]    
    for table in list_tables:
        result=detect_overlap(table,query,repet)
        if result !=[] :
            for i in range(len(result)):
                obj=Set()
                obj.setFromTuple(result[i])
                subject_list.append(obj)
            subject_list_merge=SetUtils.mergeSetsInList(subject_list)
    return subject_list_merge


def help ():
    print "usage:",sys.argv[0],"[options] "
    print "options:"
    print "    -h: this help"
    print "    -b: blast out file "
    print "    -t: TE annotations (tables Set or Path)"
    print "    -s: SAT annotations (tables Set or Path)"
    sys.exit(0)
  

def main ():
    out_blast=""
    TE_annot=[]
    SAT_annot=[]
    verbosity = 0
    
    opts,args = getopt.getopt(sys.argv[1:],"b:t:s:h")
    for o,a in opts:
        if o == "-h":
            help()
        if o == "-b":
            out_blast=a
        if o == "-t":
            TE_annot.append(a)
        if o == "-s":
            SAT_annot.append(a)
      

    if out_blast=="" or TE_annot==() or SAT_annot==():
        help()
        
   
    #######################
    # DATABASE CONNECTION #
    #######################
    repet = DbFactory.createInstance()

    dico_tables={}
    dico_tables["TE"]=TE_annot
    dico_tables["SAT"]=SAT_annot
    #print dico_tables 
            
    #############################
    # Create out table: NoRepet #
    #############################
    size=len(out_blast.split("/"))
    table_name=out_blast.split("/")[size-1].replace('-','_').replace('.','_')
    repet.createTable("%s_NoRep" % table_name, "align", overwrite = True)
#    repet.remove_if_exist(table_name+"_NoRep")
#    create='create table %s_NoRep (query_name varchar(255),query_start int(10) unsigned,\
#    query_end int(10) unsigned,subject_name varchar(255),subject_start int(10) unsigned,\
#    subject_end int(10) unsigned,E_value double,score int(10) unsigned,identity float,\
#    index query (query_name,query_start,query_end),\
#    index subject (subject_name,subject_start,subject_end));'
#    sql=create%(table_name)
#    repet.execute(sql)

    fichier=open(out_blast)
    b=Align()
    i=0
    j=0
    k=0
    
    #############################
    #    Read the match file    #
    #############################
    if verbosity > 2:
        print "begin : Total="+str(i+j+k)+"\tTotal 100%="+str(j+k)+"\tTE+SAT="+str(j)+"\tINSERT="+str(k)+"\n"
    while b.read(fichier):
#        print i,"*"*50
#        b.show()
        i=i+1
        total_over=0
        query_list=[]
        subject_list_all=[]
    
        # set list creation for the query
        newquery=Set()
        newquery.setFromTuple((-1,"",b.range_query.seqname,b.range_query.start,b.range_query.end))
        query_list.append(newquery)
        size=SetUtils.getCumulLength(query_list)
        
        # calculate the coverage rate in TE and Satellites (verif if the coverage involves only one (or two annotations because of the two tables) TE)
        subject_list_all=over_set_list(dico_tables["TE"],b.range_query,repet)

        if verbosity > 2:
            for s in subject_list_all:
                s.show()
       
        if subject_list_all!=[] and len(subject_list_all)<=len(dico_tables["TE"]):
            total_over=SetUtils.getOverlapLengthBetweenLists(subject_list_all,query_list)
            total_rate=(float(total_over)/float(size))*100  

            # If covering rate of >=95% in TE ==> detect Sat at the sequence ends
            if total_rate>=95:
                if total_rate>=99:
                    j=j+1
                    
                    if verbosity > 2:
                        print "NO INSERT...TE Coverage sup 99%"
                        b.show()
                        print "############### TOTAL==>  --OVER-- "+str(total_over)+" --RATE-- "+str(total_rate)+"##################"
            
                else:# 95%<=total_rate<99
                    for table in dico_tables["SAT"]:
                        Ttype=type_table(table,repet)
                        qcount,qpos,qsize_start,qsize_end=extract_numberRepeat_end(Ttype,table,b.range_query.seqname,b.range_query.start,b.range_query.end,repet)
                        scount,spos,ssize_start,ssize_end=extract_numberRepeat_end(Ttype,table,b.range_subject.seqname,b.range_subject.start,b.range_subject.end,repet)
                        if qcount==0 or scount==0:
                            if (size-total_over)>20:
                                if verbosity > 2:
                                    print "INSERT"
                                insert='insert delayed into %s_NoRep values ("%s",%d,%d,"%s",%d,%d,%f,%d,%f);'
                                sql=insert%(table_name,b.range_query.seqname,b.range_query.start,b.range_query.end,\
                                            b.range_subject.seqname,b.range_subject.start,b.range_subject.end,\
                                            b.e_value,b.score,b.identity)
                            
                                repet.execute(sql)
                                k=k+1
                            else:
                                j=j+1
                                
                                if verbosity > 2:
                                    print "NO INSERT...too short"
                                    b.show()
                                    print "############### TOTAL==>  --OVER-- "+str(total_over)+" --RATE-- "+str(total_rate)+"##################"
                            
                            
                        else:   
                            j=j+1
                            
                            if verbosity > 2:
                                print "NO INSERT...Satellites detected at sequence ends"
                                b.show()
                                print "############### TOTAL==>  --OVER-- "+str(total_over)+" --RATE-- "+str(total_rate)+"##################"
                            

            else:
                if verbosity > 2:
                    print "INSERT"
                insert='insert delayed into %s_NoRep values ("%s",%d,%d,"%s",%d,%d,%f,%d,%f);'
                sql=insert%(table_name,b.range_query.seqname,b.range_query.start,b.range_query.end,\
                            b.range_subject.seqname,b.range_subject.start,b.range_subject.end,\
                            b.e_value,b.score,b.identity)
               
                repet.execute(sql)
                k=k+1
                
        else:
            if verbosity > 2:
                print "INSERT"
            insert='insert delayed into %s_NoRep values ("%s",%d,%d,"%s",%d,%d,%f,%d,%f);'
            sql=insert%(table_name,b.range_query.seqname,b.range_query.start,b.range_query.end,\
                        b.range_subject.seqname,b.range_subject.start,b.range_subject.end,\
                        b.e_value,b.score,b.identity)
           
            repet.execute(sql)
            k=k+1

        if verbosity > 2:
            print "Total="+str(j+k)+"\tTE+SAT="+str(j)+"\tINSERT="+str(k)+"\n"
            
if __name__ == "__main__": 
    main ()
