#!/usr/bin/env python

"""
Last step of the Pipeline: Identification of the tandem arrays using coordinates of the repeated units. 
"""
import os
import sys
import getopt
import MySQLdb
from commons.core.coord.Set import Set
from commons.core.sql.DbMySql import DbMySql

#-----------------------------------------------------------------------------
def help ():

    """
    Give the list of the command-line options.
    """
    
    print "usage:",sys.argv[0]," [ options ]"
    print "option:"
    print "    -t: set table"
    print "    -s: max spacing value (default = 10bp)"
    print "    -c: configuration file from the SATannot pipeline"
    print "    -h: this help"
    print "    -v: verbose (0/1)"
  
#-----------------------------------------------------------------------------

def main ():

    """
    This program
    """
    
    setTable = ""
    spacing = 1000
    verbose = 0
    try:
        opts, args = getopt.getopt(sys.argv[1:],"ht:s:c:v:")
        
    except getopt.GetoptError, err:
        print str(err)
        help()
        sys.exit(1)
        
    for o, a in opts:
        if o == "-h":
            help(); sys.exit(0)
        if o == "-t":
            setTable = a
        if o == "-s":
            spacing = int(a)
        if o == "-c":
            configFileName = a
        if o == "-v":
            verbose = int(a)
        
    if  setTable == "":
        print "*** Error: missing compulsory options"
        help()
        sys.exit(1)
        
    if verbose > 0:
        print "\nbeginning of %s" % (sys.argv[0].split("/")[-1])
        sys.stdout.flush()

    if configFileName != "":
        if not os.path.exists( configFileName ):
            print "*** Error: configuration file '%s' doesn't exist" % ( configFileName )
            sys.exit(1)
    #Databases Connection
    db = DbMySql(cfgFileName = configFileName)
   
    print "Max spacing value: " + str( spacing ) + "bp"

    # Out table
    outTable = setTable + "_tandem"
    db.dropTable(outTable)
    db.createTable(outTable, "set")
   
    sqlCmd = 'select name from %s;' %( setTable )
    db.execute( sqlCmd)
    setResult = db.fetchall()
    
    #---------------------------------------------------------------------------#
    # Create the list of cluster
    print "Create cluster list... "
    cluster_list = []
    cluster_dico = {}

    cl = setResult[0][0].split("Piler_")[1]
    prefix = setResult[0][0].split("Piler_")[0]
    cluster = int(cl.split( "_" )[0])
    cluster_dico [ cluster ] = cluster 
    for R in setResult:
        cluster = R[0].split("Piler_")[1]
        cluster = int(cluster.split( "_" )[0])
        if not cluster_dico.has_key(cluster):
            cluster_dico[cluster] = cluster

    #---------------------------------------------------------------------------#
    for cluster in cluster_dico.keys():
        refChr = Set()
        chr = Set()
        select = 'select * from %s where name like "%%Piler\_%d\_%%";'
        sqlCmd = select % ( setTable, cluster)
        db.execute( sqlCmd )
        result = db.fetchall()
        print "#"*50
        print "\nCLUSTER -> " + str( cluster ) + "\tNBSEQ -> " + str( len( result ) )+"\n"
        
        #---------------------------------------------------------------------------#
        # RE-orientation + sort
        new_result = []
        for i in range( 0, len( result ) - 1 ):
            start = min( int( result[i][3] ),int( result[i][4] ) )
            end = max( int( result[i][3] ), int( result[i][4] ) )
            new_result.append( ( result[i][2], start, end, result[i][0], result[i][1] ) )
        new_result.sort()
        result = []
        for n in new_result:
            result.append( ( n[3], n[4], n[0], n[1], n[2]) )
  
        #---------------------------------------------------------------------------#
        # Calcul the distance
        dist = {}
        tr = -1
        for i in range( 0, len( result ) - 1 ):
            distance = 0
            refChr.setFromTuple( result[i] )
            if tr != -1 :
                refChr.id = tr
            newrefChr_id = refChr.id
            newrefChr_start = refChr.start
            newrefChr_end = refChr.end    
            newrefChr_seqname = refChr.seqname[0]
            chr.setFromTuple( result[i+1] )
            newchr_id = chr.id
            newchr_start = chr.start
            newchr_end = chr.end    
            newchr_seqname = chr.seqname[0]
            
            if newrefChr_seqname == newchr_seqname:
                if max( newrefChr_start, newrefChr_end ) > max( newchr_start, newchr_end):
                    if min( newrefChr_start, newrefChr_end) > max( newchr_start, newchr_end):
                        distance = min( newrefChr_start, newrefChr_end ) - max(newchr_start, newchr_end )
                    else:
                        distance = 0
                else:
                    if min( newchr_start, newchr_end ) > max( newrefChr_start, newrefChr_end):
                        distance = min( newchr_start, newchr_end ) - max( newrefChr_start, newrefChr_end )
                    else:
                        distance = 0
                         
            #---------------------------------------------------------------------------#
            # Update the path number according to the distance 
            if distance != 0:
                if distance < spacing :
                    chr.id = refChr.id
                    print "same tandem arrays #"+str(refChr.id)+" distance = "+str(distance)
                    tr = refChr.id
                else:
                    print "switch to another TR"
                    tr = -1
    
            insert = 'insert into %s values (%d,"%s","%s",%d,%d);'
            sqlCmd = insert%( outTable, chr.id, prefix+"_"+str(cluster), chr.seqname, chr.start, chr.end )
            db.execute( sqlCmd )

#--------------------------------------------------------------------------

    if verbose > 0:
        print "%s finished successfully\n" % (sys.argv[0].split("/")[-1])
        sys.stdout.flush()

    return 0

#----------------------------------------------------------------------------

if __name__ == '__main__':
    main()
