#!/usr/bin/env python

import sys
import getopt
import optparse
import os
import random
from denovo_pipe.GenomeBackboneWriter import GenomeBackboneWriter
from denovo_pipe.GenomeBackboneWriter import GenomeGrammar
from denovo_pipe.Scenario import Scenario00
from denovo_pipe.Scenario import Scenario01
from commons.core.seq.Bioseq import Bioseq
from denovo_pipe.DawgLauncher import DawgLauncher
from commons.core.coord.Align import Align 


class GenomeSimulator( object ):
    
    def __init__(self):
        self._scenario = -1
        self._teFile = ""
        self._seed = 1859
        self._seedForTransitionProbabilites = None
        self._nbTEs = -1
        self._dawgConfigFile = ""
        self._outPrefix = ""
        self._clean = False
        self._verbose = 0
        self._rnd = random
        self._teCopiesPrefix = ""
        #self._lengthOther = 2000
        self._lengthOther = 20
        self._iBsDbGene = None  # BioseqDB() with the input gene file
        self._countGene = 0
        self._iBsDbTe = None  # BioseqDB() with the TE copies generated by DAWG
        self._countTe = 0
        self._iScenario = None        
        self._genomeGrammarConfigFile = ""        
       	self._dOtherIndex = {}

    def help(self):
        print
        print "usage: %s.py [options]" % ( self.__class__.__name__ )
        print "options:"
        print "     -h: this help"
        print "     -s: scenario"
        print "         00: burst, one TE family with >3 exact similar copies, no genes. Use this scenario for control"
        print "         01: burst, one TE family with divergent copies, no genes. "
        print "     -S: seed for backbone generation (default=1859)"
        print "     -r: seed for transition probabilites generation for Scenario 01"
        print "     -n: nb TEs copies in genome"
        print "     -d: configuration file for DAWG (can be used with scenario 1)"
        print "     -c: clean"
        print "     -v: verbosity level (default=0/1/2)"
        print
        
        
    def setAttributesFromCmdLine(self):
        examples = ""
        scenarioHelp = ""
        parser = optparse.OptionParser(description= "GenomeSimulator", version = "v1.0", epilog=examples)
        parser.add_option( "-s", "--scenario", dest = "scenario", help="")
        try:
            opts, args = getopt.getopt( sys.argv[1:], "hs:t:g:S:n:r:d:o:cp:i:v:" )
        except getopt.GetoptError, err:
            msg = str(err)
            sys.stderr.write( "%s\n" % msg )
            self.help()
            sys.exit(1)
        for o,a in opts:
            if o == "-h":
                self.help(); sys.exit(0)
            elif o =="-s":
                self._scenario = a
            elif o == "-t":
                self._teFile = a
            elif o == "-S":
                self._seed = int(a)
            elif o == "-n":
                self._nbTEs = int(a)
            elif o == "-r":
                self._seedForTransitionProbabilites = int(a)
            elif o == "-d":
                self._dawgConfigFile = a
            elif o == "-c":
                self._clean = True
            elif o == "-v":
                self._verbose = int(a)
                
                
    def checkAttributes(self):
        if self._scenario == -1:
            msg = "ERROR: missing scenario (-s)"
            sys.stderr.write( "%s\n" % msg )
            self.help()
            sys.exit(1)
        if self._scenario in (00, 01) and self._teFile != "":
            msg = "ERROR: scenario %i does not require a file with TEs" % ( self._scenario )
            sys.stderr.write( "%s\n" % msg )
            self.help()
            sys.exit(1)
        if self._scenario in (01,) and self._dawgConfigFile != "":
            msg = "ERROR: scenario %i does require a dawgCofigFile" % ( self._scenario )
            sys.stderr.write( "%s\n" % msg )
            self.help()
            sys.exit(1)
        if self._scenario in (01,) and self._seedForTransitionProbabilites != "":
            msg = "ERROR: scenario %i does require a seed for transition probabilities generation, r option " % ( self._scenario )
            sys.stderr.write( "%s\n" % msg )
            self.help()
            sys.exit(1)
        if self._scenario in (01,) and self._nbTEs != -1:
            msg = "ERROR: scenario %i does require nb TEs copies, n option " % ( self._scenario )
            sys.stderr.write( "%s\n" % msg )
            self.help()
            sys.exit(1)
 
        if self._dawgConfigFile != "" and not os.path.exists( self._dawgConfigFile ):
            msg = "ERROR: can't find file '%s'" % ( self._dawgConfigFile )
            sys.stderr.write( "%s\n" % msg )
            self.help()
            sys.exit(1)
        if self._outPrefix == "":
            self._outPrefix = "GenSimS%sSeed%s" % ( self._scenario, self._seed )
    
    def setScenario(self, iScenario):
	self._iScenario = iScenario    
            
    
    def createOtherSequence(self):
        lNucleotides = ["A","G","C","T"]
	otherSequence = ""
        for i in xrange( 0, self._lengthOther ):
            otherSequence += self._rnd.choice(lNucleotides)
        return otherSequence   

    # TODO now this method is delegated to _GeneSequence, write test on this class    
    def setGeneSequence(self, genome):
	iGeneSequence = _GeneSequence()
	iGeneSequence.setDataSource(self._iBsDbGene)
	iGeneSequence.createSequence(None, genome)	
        self._countGene = iGeneSequence._countSequence
        
	        
    # TODO now this method is delegated to _TeSequence, write test on this class    
    def setTeCopySequence(self, teBs, genome):
	iTeSequence = _TeSequence()
	iTeSequence.setDataSource(self._iBsDbTe)
	iTeSequence.createSequence(None, genome)	
        self._countTe = iTeSequence._countSequence
        
    def setTeCopyCoord(self, teBs, iAlign, currentPosition):
        #iAlign.reset()
        #iAlign.setFromTuple(("chr",currentPosition, currentPosition + teBs.getLength(), teBs.getHeader(), 1, teBs.getLength(), 0, 0, 0 ))
     	iTeSequence = _TeSequence()
        self._countTe = iTeSequence._countSequence
    	iTeSequence._tmpBs = teBs
	iTeSequence.setCoord(currentPosition)
	tmpAlign = iTeSequence._lCoord[0]
	iAlign.setQueryName(tmpAlign.getQueryName())
	iAlign.setQueryStart(tmpAlign.getQueryStart()) 
	iAlign.setQueryEnd(tmpAlign.getQueryEnd()) 
	iAlign.setSubjectName(tmpAlign.getSubjectName()) 
	iAlign.setSubjectStart(tmpAlign.getSubjectStart()) 
	iAlign.setSubjectEnd(tmpAlign.getSubjectEnd()) 


    def _createOtherSequence(self):
        lNucleotides = ["A","G","C","T"]
	otherSequence = ""
        for i in xrange( 0, self._lengthOther ):
            otherSequence += self._rnd.choice(lNucleotides)
        return otherSequence   

    def generateWholeGenomeSequence(self, iBackbone, filePrefix, iTeBs=None, iGeneBs=None):
        outFastaFile = "%s.fa" % ( filePrefix )
        outFastaFileHandler = open( outFastaFile, "w" )
        outFastaFileHandler.write( ">chr\n" )
        
        outAlignFile = "%s.align" % ( filePrefix )
        outAlignFileHandler = open( outAlignFile, "w" )
	
	iGenome = Genome()	

	genome, lTeCoord = iGenome.generateGenomeAndTeCoord(iBackbone, self, iTeBs, iGeneBs)        
	# write genome
	lineLength = 0
        for i in xrange( 0, genome.getLength() ):
                outFastaFileHandler.write(genome.sequence[i])
                lineLength += 1
                if lineLength == 60:
                    outFastaFileHandler.write( "\n" )
                    lineLength = 0
                    
        if lineLength < 60:
            outFastaFileHandler.write( "\n" )

	# write TeCoord
 	for iAlign in lTeCoord:
		iAlign.write(outAlignFileHandler)
	
	outFastaFileHandler.close()
        outAlignFileHandler.close()
        
        os.system( "align2set.py -i %s" % outAlignFile )
       
        if self._clean:
            if self._isScenarioEnabled():
                os.remove("%s.fa" % self._iScenario._teCopiesPrefix)
            else:    
                os.remove( "%s.fa" % self._teCopiesPrefix )
                
        if self._verbose > 0:
            if lengthGenome > 0:
                msg = "genome length: %i bp (%.0f%% TEs)" % ( lengthGenome, 100*lengthTEs/float(lengthGenome) )
                sys.stdout.write( "%s\n" % msg )
                sys.stdout.flush()
                
                
    def start(self):
        self.checkAttributes()
        self._rnd.seed( self._seed )
        if self._verbose > 0:
            msg = ""
            msg += "START %s.py" % ( self.__class__.__name__ )
            if self._scenario != -1:
                msg += "\nscenario: %s" % ( self._scenario )
            if self._dawgConfigFile != "":
                msg += "\nDAWG config: %s" % ( self._dawgConfigFile )
            msg += "\nseed: %i" % ( self._seed )
            if self._teFile != "":
                msg += "\nTE file: %s" % ( self._teFile )
            sys.stdout.write( "%s\n" % msg )
            sys.stdout.flush()
        
    def run(self):
        self.start()
	 
        iScenario =self._createScenario()
	iGenome = Genome(iScenario)
	iBackbone = iGenome.generateGenomeBackbone(iScenario, self._seed, self._seedForTransitionProbabilites, self._i._nbTEs)
        iTeBs = self._iScenario.generateTEs(iBackbone.count("t"))
        self.generateWholeGenomeSequence(iBackbone, self._i._outPrefix)
        
        if self._verbose > 0:
            msg = "END %s.py" % ( self.__class__.__name__ )
            sys.stdout.write( "%s\n" % msg )
            sys.stdout.flush()
            
            
    def _isScenarioEnabled(self):
        return self._iScenario != None   
    
    
    def _createScenario(self):
	iScenario = None
        if self._scenario == "00":
            iScenario = Scenario00()
            iScenario.setClean(self._clean) 
            iScenario.setVerbose(self._verbose) 
        if self._scenario == "01":
            iScenario = Scenario01()
            iScenario.setDawgConfigFile(self._dawgConfigFile)
            iScenario._verbose = self._verbose
	return iScenario

class Genome:
    def generateGenomeBackbone(self, iScenario, seedForBackboneGeneration, seedForTransitionProbabilities, nbTEs):
        iScenario.setSeedForRandomizeProbas(seedForTransitionProbabilities)
        iScenario.setNbTe(nbTEs)
        iScenario.setSeedForBackboneGeneration(seedForBackboneGeneration)
        iBackbone = iScenario.generateGenomeBackbone() 
        return iBackbone

    def generateGenomeAndTeCoord(self, backbone, iGenomeSimulator=None, teBioseq=None, geneBioseq=None):
	genome = Bioseq()
  	lengthGenome = 0
	
	iOtherSequence = _OtherSequence(backbone)
	iGeneSequence = _GeneSequence()
	iGeneSequence.setDataSource(geneBioseq)
	
	iTeSequence = _TeSequence()
	iTeSequence.setDataSource(teBioseq)

	currentInstanceSequence = None

	for sample in backbone:
	    currentPosition = lengthGenome
            if sample == "o":
		currentInstanceSequence = iOtherSequence
		iGs = self
	    elif sample == "g":
		currentInstanceSequence = iGeneSequence
		iGs = None
            elif sample == "t":
		currentInstanceSequence = iTeSequence
		iGs = None
            currentInstanceSequence.createSequence(iGenomeSimulator, genome)
	    currentInstanceSequence.setCoord(currentPosition)     

	    lengthGenome = genome.getLength()
	lTeCoord = iTeSequence.getLCoord()
        return genome, lTeCoord
	
    def generateWholeGenomeSequence(self, iGenomeSimulator, iBackbone, filePrefix, iTeBs=None, iGeneBs=None):
        outFastaFile = "%s.fa" % ( filePrefix )
        outFastaFileHandler = open( outFastaFile, "w" )
        outFastaFileHandler.write( ">chr\n" )
        
        outAlignFile = "%s.align" % ( filePrefix )
        outAlignFileHandler = open( outAlignFile, "w" )
	
	iGenome = Genome()	

	genome, lTeCoord = iGenome.generateGenomeAndTeCoord(iBackbone, iGenomeSimulator, iTeBs, iGeneBs)        
	# write genome
	lineLength = 0
        for i in xrange( 0, genome.getLength() ):
                outFastaFileHandler.write(genome.sequence[i])
                lineLength += 1
                if lineLength == 60:
                    outFastaFileHandler.write( "\n" )
                    lineLength = 0
                    
        if lineLength < 60:
            outFastaFileHandler.write( "\n" )

	# write TeCoord
 	for iAlign in lTeCoord:
		iAlign.write(outAlignFileHandler)
	
	outFastaFileHandler.close()
        outAlignFileHandler.close()
        
        os.system( "align2set.py -i %s" % outAlignFile )
       
        if iGenomeSimulator._clean:
            if iGenomeSimulator._isScenarioEnabled():
                os.remove("%s.fa" % iGenomeSimulator._iScenario._teCopiesPrefix)
            else:    
                os.remove( "%s.fa" % iGenomeSimulator._teCopiesPrefix )
                
        if iGenomeSimulator._verbose > 0:
            if lengthGenome > 0:
                msg = "genome length: %i bp (%.0f%% TEs)" % ( lengthGenome, 100*lengthTEs/float(lengthGenome) )
                sys.stdout.write( "%s\n" % msg )
                sys.stdout.flush()
 

class _OtherSequence:

    def __init__(self, iBackbone):
	self._iBackbone = iBackbone
	self._dOtherIndex = {}

    def createSequence(self, iGenomeSimulator, genome): 
	otherCurrentIndexInBackbone = self._iBackbone.getIndex()
	if not iGenomeSimulator._dOtherIndex.has_key(otherCurrentIndexInBackbone):
		otherSequence = iGenomeSimulator._createOtherSequence()
	        iGenomeSimulator._dOtherIndex[otherCurrentIndexInBackbone] = otherSequence
	genome.sequence += iGenomeSimulator._dOtherIndex[otherCurrentIndexInBackbone]	
    
    def setCoord(self, currentPosition):
	pass
  

  
class _GeneSequence:

    def __init__(self):
	self._dataSource = None
	self._countSequence = 0

    def setDataSource(self, dataSource):
	self._dataSource = dataSource

    def createSequence(self, iGenomeSimulator, genome):
	if self._countSequence == self._dataSource.getSize():
		self._countSequence = 0
	genome.sequence += self._dataSource.db[self._countSequence].sequence
        self._countSequence += 1
        
    def setCoord(self, currentPosition):
	pass

class _TeSequence:
	
    def __init__(self):
	self._tmpBs = Bioseq()
	self._dataSource = None
	self._lCoord = []
	self._countSequence = 0

    def getLCoord(self):
	return self._lCoord
     
    def setDataSource(self, dataSource):
	self._dataSource = dataSource
	
    def createSequence(self, iGs, genome):
	self._tmpBs.reset()
	if self._countSequence == self._dataSource.getSize():
            	self._countSequence = 0
	self._tmpBs.sequence = self._dataSource.db[ self._countSequence ].sequence
        self._tmpBs.header = self._dataSource.db[ self._countSequence ].header
	
	genome.header = None
	genome.sequence += self._tmpBs.sequence
	self._countSequence += 1

    def setCoord(self, currentPosition):
	iAlign = Align()
        iAlign.setFromTuple(("chr",currentPosition, currentPosition + self._tmpBs.getLength(), self._tmpBs.getHeader(), 1, self._tmpBs.getLength(), 0, 0, 0 )) 
	self._lCoord.append(iAlign)

if __name__ == "__main__":
    i = GenomeSimulator()
    i.setAttributesFromCmdLine()
    i.run()
