# VGBrowse : Vectorial Genome Browser
#
#Copyright (C) 2006-2007, Charles Vejnar <charles [AT] vejnar.eu>
#
#This file is part of VGBrowse.
#
#VGBrowse is free software; you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation; either version 2 of the License, or
#(at your option) any later version.
#
#VGBrowse is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with VGBrowse; if not, write to the Free Software
#Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

from pyRepet.coord.Range import *
from pyRepet.coord.Set import *
from SVGdraw import *

class RangeFeature(Range):
	"""
	Record a annotation, attribute(s) and sub-annotation(s)

	@param sub_rangefeatures: the features under the region
	@type sub_rangefeatures: list

	@param attributes: the attributes of the region
	@type attributes: dict

	@param id: identifier number
	@type id: integer

	@param parent: parent identifier number
	@type parent: integer

	@param type: the type of the region
	@type type: string

	@param name: the name of the region
	@type name: string

	@ivar seqname: the sequence name
	@type seqname: string

	@ivar start: start coordinate
	@type start: integer

	@ivar end: end coordinate
	@type end: integer

	@note: on reverse strand if start>end
	"""

	def __init__(self,sub_rangefeatures=None,set_list=None,attributes=None,id=-1,parent=-1,type='',name='',seqname='',start=-1,end=-1):
		"""
		constructor
	
		@param sub_rangefeatures: the features under the region
		@type sub_rangefeatures: list
	
		@param attributes: the attributes of the region
		@type attributes: dict
	
		@param id: identifier number
		@type id: integer
	
		@param parent: parent identifier number
		@type parent: integer
	
		@param type: the type of the region
		@type type: string
	
		@param name: the name of the region
		@type name: string
	
		@param seqname: the sequence name
		@type seqname: string
	
		@param start: start coordinate
		@type start: integer
	
		@param end: end coordinate
		@type end: integer
		"""
		Range.__init__(self,seqname,start,end)
		if sub_rangefeatures is None:
			sub_rangefeatures=[]
		if set_list is None:
			set_list=[]
		if attributes is None:
			attributes={}
		self.name=name
		self.type=type
		self.parent=parent
		self.id=id
		self.attributes=attributes
		self.sub_rangefeatures=sub_rangefeatures
		self.set_list=set_list

	def drawSVG(self,csvg,label,add_top_point):
		if self.set_list==[]:
			#print csvg.top_point,self.name
			if label:
				csvg.svgelement_list.append(text(csvg.left_margin,csvg.top_point+20,self.type,text_anchor='start',id='label'))
				csvg.svgelement_list.append(text(csvg.left_margin+csvg.type_margin,csvg.top_point+40,self.name,text_anchor='start',id='label'))
			glyph_start=csvg.left_margin+csvg.type_margin+int(self.start-csvg.x_ref)/csvg.scale
			glyph_end=csvg.left_margin+csvg.type_margin+int(self.end-csvg.x_ref)/csvg.scale
			glyph_position=45
			glyph_round=8
			glyph_arrow=10
			glyph_height=20
			if glyph_round+glyph_arrow<=glyph_end-glyph_start:
				csvg.svgelement_list.append(path(pathdata='M %i,%i L %i,%i L %i,%i L %i,%i L %i,%i S %i,%i %i,%i L %i,%i S %i,%i %i,%i z'%(glyph_start+glyph_round,csvg.top_point+glyph_position,glyph_end-glyph_arrow,csvg.top_point+glyph_position,glyph_end,csvg.top_point+glyph_position+glyph_height/2,glyph_end-glyph_arrow,csvg.top_point+glyph_position+glyph_height,glyph_start+glyph_round,csvg.top_point+glyph_position+glyph_height,glyph_start,csvg.top_point+glyph_position+glyph_height,glyph_start,csvg.top_point+glyph_position+glyph_height-glyph_round,glyph_start,csvg.top_point+glyph_position+glyph_round,glyph_start,csvg.top_point+glyph_position,glyph_start+glyph_round,csvg.top_point+glyph_position),fill='#7b8de7',stroke='black',stroke_width='1'))
			else:
				csvg.svgelement_list.append(polygon([[glyph_start,csvg.top_point+glyph_position],[glyph_end,csvg.top_point+glyph_position+glyph_height/2],[glyph_start,csvg.top_point+glyph_position+glyph_height]],stroke='black',stroke_width='1',fill='#7b8de7'))
			csvg.top_point+=add_top_point
			list_types=[]
			list_operation=[]
			for rangefeature in self.sub_rangefeatures:
				list_types.append(rangefeature.type)
			#print 'LIST sub rf : %s'%list_types
			for type in csvg.list_order:
				#print 'TYPE config : '+type
				i=-1
				try:
					while 1:
						i=list_types.index(type, i+1)
						#print "match at",i
						list_operation.append([i,True,csvg.inter_track_space])
				except ValueError:
					pass
			for i in xrange(len(list_types)):
				switch=True
				for op in list_operation:
					if op[0]==i:
						switch=False
				if switch:
					list_operation.append([i,True,csvg.inter_track_space])
					if list_types[i] in csvg.list_together:
						for j in xrange(i+1,len(list_types)):
							if list_types[j] in csvg.list_together:
								list_operation.append([j,True,csvg.inter_track_space])
			i=0
			while i<len(list_operation):
				op=list_types[list_operation[i][0]]
				if op in csvg.list_together:
					list_operation[i][1]=True
					list_operation[i][2]=0
					i+=1
					while i<len(list_operation) and op==list_types[list_operation[i][0]]:
						list_operation[i][1]=False
						list_operation[i][2]=0
						i+=1
					i-=1
					list_operation[i][1]=False
					list_operation[i][2]=csvg.inter_track_space
				i+=1
#			print 'ope : %s'%list_operation
			for op in list_operation:
				self.sub_rangefeatures[op[0]].drawSVG(csvg,op[1],op[2])
		else:
			#print csvg.top_point,self.name
			csvg.svgelement_list.append(text(csvg.left_margin,csvg.top_point+30,self.type,text_anchor='start',id='label'))
			for list_set in set_list_split_by_name(self.set_list).values():
				list_set_length=len(list_set)
				csvg.svgelement_list.append(text(csvg.left_margin+csvg.type_margin,csvg.top_point+40,self.name,text_anchor='start',id='label'))
				for rangefeature in self.sub_rangefeatures:
					csvg.svgelement_list.append(rect(x=csvg.left_margin+csvg.type_margin+(rangefeature.start-csvg.x_ref)/csvg.scale,y=csvg.top_point+45,width=(rangefeature.end-rangefeature.start)/csvg.scale,height='12',fill='#d589e0',stroke='black',stroke_width='1',rx='3',ry='3'))
				for i in xrange(list_set_length):
					set=list_set[i]
					start_x0=csvg.left_margin+csvg.type_margin+(set.start-csvg.x_ref)/csvg.scale
					end_x0=start_x0+(set.end-set.start)/csvg.scale
					csvg.svgelement_list.append(rect(x=start_x0,y=csvg.top_point+45,width=(set.end-set.start)/csvg.scale,height='12',fill='#ffa200',stroke='black',stroke_width='1',rx='3',ry='3'))
					if i<list_set_length-1:
						start_x1=csvg.left_margin+csvg.type_margin+(list_set[i+1].start-csvg.x_ref)/csvg.scale
						csvg.svgelement_list.append(path('M %i %i Q %i %i %i %i'%(end_x0,csvg.top_point+50,end_x0+(start_x1-end_x0)/2,csvg.top_point+30,start_x1,csvg.top_point+50),fill='none',stroke='blue',stroke_width='1.5'))
				csvg.top_point+=40

	def genSVG_alone(self,left_margin,type_margin,inter_track_space,scale,x_ref,list_order,list_together):
		class contentSVG(object):
			def __init__(self,width=0,left_margin=0,type_margin=0,inter_track_space=0,scale=0,top_point=0,x_ref=0,svgelement_list=None,list_order=None,list_together=None):
				if svgelement_list is None:
					svgelement_list=[]
				if list_order is None:
					list_order=[]
				if list_together is None:
					list_together=[]
				self.width=width
				self.left_margin=left_margin
				self.type_margin=type_margin
				self.inter_track_space=inter_track_space
				self.scale=scale
				self.top_point=top_point
				self.x_ref=x_ref
				self.svgelement_list=svgelement_list
				self.list_order=list_order
				self.list_together=list_together
			def drawCompleteSVG(self):
				d=drawing()
				s=svg(width=str(self.width)+'px')
				for element in self.svgelement_list:
					s.addElement(element)
				s.attributes['height']=str(self.top_point+30)+'px' # Value 30 is empirical : top_point+30 may be the bottom point
				d.setSVG(s)
				return d.toXml()
		csvg=contentSVG(width=(self.end-self.start)/scale+left_margin+type_margin+1,left_margin=left_margin,type_margin=type_margin,inter_track_space=inter_track_space,scale=scale,x_ref=x_ref,list_order=list_order,list_together=list_together)
		self.drawSVG(csvg,True,csvg.inter_track_space)
		return csvg.drawCompleteSVG()
