import os
import unittest
from LauncherBlasterNotInParallel import *


class TestLauncherBlasterNotInParallel(unittest.TestCase):
  
    def setUp(self):
        self.launcher = LauncherBlasterNotInParallel()    
   
    def testCheckSubjectInputSequence(self):
        # atomic
        dummyQueryInputSeq = open("dummySubject.fa", "w")
        dummyQueryInputSeq.write("line1\n")
        dummyQueryInputSeq.write("line2\n")
        dummyQueryInputSeq.write("line3")
        dummyQueryInputSeq.close()
        self.launcher.setSubjectInputSequence("dummySubject.fa");
        self.assertEqual(1,self.launcher.checkSubjectInputSequence())
        os.system("rm dummySubject.fa")
        # file empty
        dummyQueryInputSeq = open("dummySubject.fa", "w")
        dummyQueryInputSeq.close()
        self.launcher.setSubjectInputSequence("dummySubject.fa");
        self.assertEqual(0,self.launcher.checkSubjectInputSequence())
        os.system("rm dummySubject.fa")
        #file does not exist
        self.launcher.setSubjectInputSequence("dummySubject.fa");
        self.assertEqual(0,self.launcher.checkSubjectInputSequence())
    
    
    def testCheckQueryInputFolder(self):
        # folder with 1 file
        os.mkdir("dummyFolder")
        self.launcher.setQueryInputFolder("dummyFolder");
        os.system("touch dummyFolder/dummySubject1.fa")
        
        dummyQueryInputSeq = open("dummyFolder/dummySubject1.fa", "w")
        dummyQueryInputSeq.write("line1\n")
        dummyQueryInputSeq.write("line2\n")
        dummyQueryInputSeq.write("line3")
        dummyQueryInputSeq.close()
   
        self.assertEqual(1,self.launcher.checkQueryInputFolder())
        # folder with 2 file
        os.system("touch dummyFolder/dummySubject2.fa")
        dummyQueryInputSeq = open("dummyFolder/dummySubject2.fa", "w")
        dummyQueryInputSeq.write("line1\n")
        dummyQueryInputSeq.write("line2\n")
        dummyQueryInputSeq.write("line3")
        dummyQueryInputSeq.close()
        self.assertEqual(1,self.launcher.checkQueryInputFolder())
        
        os.system("rm dummyFolder/dummySubject1.fa")
        os.system("rm dummyFolder/dummySubject2.fa")
        
        
        # folder with 1 empty files 
        os.system("touch dummyFolder/dummySubject1.fa")
        dummyQueryInputSeq = open("dummyFolder/dummySubject1.fa", "w")
        dummyQueryInputSeq.write("line1\n")
        dummyQueryInputSeq.write("line2\n")
        dummyQueryInputSeq.write("line3")
        dummyQueryInputSeq.close()
        # second file is empty
        os.system("touch dummyFolder/dummySubject2.fa")
        self.assertEqual(0,self.launcher.checkQueryInputFolder())
        os.system("rm dummyFolder/dummySubject1.fa")
        os.system("rm dummyFolder/dummySubject2.fa")
        
        # empty folder file
        self.assertEqual(0,self.launcher.checkQueryInputFolder())
        os.system("rm -r dummyFolder")
        # folder don't even exists !
        self.assertEqual(0,self.launcher.checkQueryInputFolder())
    
    def testCheckOuputFileList (self):
        # atomic
        dummyOutputSeq = open("dummyOutput1.align", "w")
        dummyOutputSeq.close()
        dummyOutputSeq = open("dummyOutput2.align", "w")
        dummyOutputSeq.close()
        dummyOutputSeq = open("dummyOutput3.align", "w")
        dummyOutputSeq.close()
        dummyOutputSeq = open("dummyOutput4.align", "w")
        dummyOutputSeq.close()
        
        self.launcher.addOutputFile("dummyOutput1.align")
        self.launcher.addOutputFile("dummyOutput2.align")
        self.launcher.addOutputFile("dummyOutput3.align")
        self.launcher.addOutputFile("dummyOutput4.align")
        self.assertEqual(1,self.launcher.checkOutputFileList())
        #file does not exist
        os.system("rm dummyOutput1.align")
        os.system("rm dummyOutput2.align")
        os.system("rm dummyOutput4.align")
        self.assertEqual(0,self.launcher.checkOutputFileList())
    
    def testProgram(self):
        # atomic
        dummyProgram = open("dummyProgram", "w")
        dummyProgram.close()
        self.launcher.setProgram("dummyProgram")
        self.assertEqual(1,self.launcher.checkProgram())
        os.system("rm dummyProgram")
        #program does not exist
        self.launcher.setProgram("dummyProgram");
        self.assertEqual(0,self.launcher.checkProgram())
        
    
if __name__ == "__main__":
	unittest.main()        
