#!/usr/bin/env python

from math import *
#import biggles
from pyRepet.seq.Bioseq import Bioseq
from pyRepet.seq.BioseqDB import BioseqDB

#------------------------------------------------------------------------------

class AlignedBioseqDB( BioseqDB ):

    #--------------------------------------------------------------------------

    def __init__(self):

        BioseqDB.__init__(self)
        self.score = 0

    #--------------------------------------------------------------------------

    def length(self):

        return self.db[0].length()
    
    #--------------------------------------------------------------------------

    def get_sub_profil( self, start, end=0 ):

        sub_profil = AlignedBioseqDB()
        for bs in self.db:
            sub = bs.subseq(start,end)
            sub.header = sub.header.split(" ",1)[0]
            sub_profil.add( sub )
        sub_profil.score = self.score
        return sub_profil

    #--------------------------------------------------------------------------

    def makeCons( self, min_base ):

        pos_count=[]
        n=0
        first=1
        for each_alSeq in self.db:
            if each_alSeq.sequence==None:
                break
            n=n+1

            if first:
                for i in xrange(0,len(each_alSeq.sequence)):
                    pos_count.append({})
                first=0
            for i in xrange(0,len(each_alSeq.sequence)):
                nuc=each_alSeq.sequence[i].upper()
                if pos_count[i].has_key(nuc):
                    pos_count[i][nuc]=pos_count[i][nuc]+1
                else:
                    pos_count[i][nuc]=1

        consensus=""
        for i in pos_count:
            max=0
            nuc='-'
            nb_nuc=0
            for j in i.keys():
                if j!='-':
                    nb_nuc=nb_nuc+i[j]
                    if i[j]>max:
                        max=i[j]
                        nuc=j
            if nb_nuc!=0 and float(max)/nb_nuc < 0.5:
                nuc='N'
           
            if int(nb_nuc) >= int(min_base):
                consensus=consensus+nuc
        cons=Bioseq()
        cons.sequence=consensus
        cons.header=self.name+" consensus length="+str(len(consensus))
        return cons

    #--------------------------------------------------------------------------

    def __calcul_log( self, sum, nb_seq ):

        if sum==0:
            return 0.
        else:
            f = float(sum) / nb_seq
            return - f * log(f) / log(2) 
    
    #--------------------------------------------------------------------------

    def identity(self):

        datay=[]
        #boucle sur la longueur de l'alignement
        for posx in xrange (0,self.length()):
            nbOfIdNuc=0
            nbOfPairs=0
            #boucle sur le nombre de sequences
            for seq in xrange(0,self.size()):
               #on stocke la lettre a comparer
               ref=self.db[seq].sequence[posx]
               #boucle sur les sequences suivantes
               for seq_suiv in xrange(seq+1,self.size()):
                   #comparaison des deux bases si identique et pas gap on incremente
                   if (self.db[seq_suiv].sequence[posx]!='-' and ref !='-'):
                       nbOfPairs+=1
                       if (self.db[seq_suiv].sequence[posx]==ref):
                           nbOfIdNuc+=1

            if nbOfPairs!=0:
                datay.append(nbOfIdNuc/nbOfPairs*100)
            else:
                datay.append(0)
        return datay

    #--------------------------------------------------------------------------

    def entropy( self ):

        datay=[]
        for posx in xrange(0,self.length()):
              nbTotal=0
              nbOfA=0
              nbOfC=0
              nbOfG=0
              nbOfT=0
              currentNuc=""
              for seq in xrange(0,float(self.size())):
                  currentNuc=self.db[seq].sequence[posx]
                 
                  if  currentNuc!="-":
                      nbTotal+=1  
                  if  currentNuc=="A" or currentNuc=="a":
                      nbOfA+=1
                  elif  currentNuc=="T" or currentNuc=="t":
                      nbOfT +=1
                  elif currentNuc=="G" or currentNuc=="g":
                      nbOfG+=1
                  elif  currentNuc=="C" or currentNuc=="c":
                      nbOfC+=1
                  elif currentNuc=="N" or currentNuc=="n":
                      nbOfA+=0.25
                      nbOfC+=0.25
                      nbOfG+=0.25

              datay.append(self.__calcul_log(nbOfA,nbTotal)+self.__calcul_log(nbOfT,nbTotal)+self.__calcul_log(nbOfC,nbTotal)+self.__calcul_log(nbOfG,nbTotal))
   
        return datay

    #--------------------------------------------------------------------------

    #liste des y pour le nombre de nucleotides a cette pos de l'alignement

    def nbNuc( self ):

        datay=[]
        for posx in xrange(0,self.length()):
            nbnuc=0 
            for seq in xrange(0,float(self.size())):
                if self.db[seq].sequence[posx]!='-':
                    nbnuc+=1
            datay.append(float(nbnuc)/self.size())
        return datay
    
##     def draw_align(self,title):
##         frame=biggles.FramedPlot()
##         frame.xlabel="$%s$"%"sequence"
##         frame.ylabel="$%s$"%"names"
##         frame.y1.ticklabels_style["fontsize"]=0.5
##         frame.x.ticklabels_style["fontsize"]=0.5
##         ylabel=[]
##         data=[]
##         for s in xrange(0,self.size()):
##             start=-1
##             end=-1
##             lseg=[]
##             BSseq=self.__getitem__(s).sequence
##             BSname=self.__getitem__(s).header
##             for i in xrange(0,self.length()):
##                 if BSseq[i]!='-':
##                     end=i
##                     if start==-1:
##                         start=i
##                 else:
##                     if start!=-1 and end!=-1:
##                         lseg.append([start+1,end+1])
##                     start=-1
##                     end=-1

##             if start!=-1 and end!=-1:
##                 lseg.append([start+1,end+1])
##             data.append((BSname,lseg))
##             data.sort()

##         for i in data:
##             ylabel.append(i[0])
##         frame.y1.ticklabels=ylabel
##         frame.y1.ticks=range(1,len(data)+1)
##         frame.y1.draw_subticks=0
##         frame.y2.ticks=range(1,len(data)+1)
##         frame.y2.draw_subticks=0
##         frame.title=title
##         y=0
##         for k in data:
##             y=y+1
##             for i in k[1]:
##                 if len(i)>2:
##                     if i[2]>0:
##                         seg=biggles.Curve(i[:2],[y,y],linewidth=5,\
##                                           linetype="solid")
##                         extL=biggles.Point(i[0],y,symboltype="|")
##                         frame.add(extL)
##                         extR=biggles.Point(i[1],y,\
##                                            symboltype=">",\
##                                            symbolesize=3)
##                         frame.add(extR)
##                     else:
##                         seg=biggles.Curve(i[:2],[y,y],linewidth=5,\
##                                           linetype="solid")
##                         extL=biggles.Point(i[0],y,\
##                                            symboltype="<",\
##                                            symbolsize=3)
##                         frame.add(extL)
##                         extR=biggles.Point(i[1],y,symboltype="|")
##                         frame.add(extR)
##                 else:
##                     seg=biggles.Curve(i,[y,y],linewidth=5,linetype="solid")
##                 frame.add(seg)
##         return frame

## ######################################################
## #dessiner un graphe a partir d'une liste de valeurs y#
## ######################################################
##     def __plot_curve(self,datax,datay,ylabel="",title=""):
##         frame=biggles.FramedPlot()
##         frame.title=title
##         frame.xlabel="sequence"
##         frame.ylabel=ylabel
##         frame.x1.ticklabels_style["fontsize"]=0.5
##         frame.add(biggles.Curve(datax,datay))
##         return frame

##     def draw_multigraph(self):
##         datax=[]
##         for x in xrange(0,self.length()):
##             datax.append(x)
##         frames=[]
##         frames.append(self.draw_align("Sequences alignment"))
##         frames.append(self.__plot_curve(datax,self.nbNuc(),"%of Nucleotides",""))
##         frames.append(self.__plot_curve(datax,self.identity(),"%identity",""))
##         frames.append(self.__plot_curve(datax,self.entropy(),"Entropy value",""))
       
        
##         nb_frames=len(frames)
##         table=biggles.Table(nb_frames,1)
##         for i in xrange(0,nb_frames):
##                       table[i,0]=frames[i]
##         table.write_eps(self.name+'.eps')
   

    #--------------------------------------------------------------------------

    def __lisse_curve(self,data,win):
        deb=win/2
        fin=len(data)-(win/2)
        liste_tmp=data[:]
        liste_tmp[:deb]=[ 0 ]*deb
        if win>0:
            liste_tmp[-fin:]=[ 0 ]*fin
        for i in xrange(deb,fin):
            for j in xrange(-win/2,win/2):
                liste_tmp[i]+=data[i+j]
            liste_tmp[i]/=float(win)
        return liste_tmp
